@@ -10,7 +10,7 @@
     \author     Robert O'Connor
     \date       2002/01/03    
  */  
-// RCS-ID:      $Id: plucker_defines.h,v 1.38.2.3 2004/01/06 13:45:50 nordstrom Exp $
+// RCS-ID:      $Id: plucker_defines.h,v 1.43 2004/03/21 14:10:30 nordstrom Exp $
 //----------------------------------------------------------------------------------------
 
 //----------------------------------------------------------------------------------------
@@ -59,7 +59,7 @@
     Rebuild configuration class and main_frame class for it to update.
     First three digits are the Plucker version, last one is any Plucker Desktop updates.
  */
-#define plkrPLUCKER_DESKTOP_VERSION_STRING          "1.6.2.0"
+#define plkrPLUCKER_DESKTOP_VERSION_STRING          "1.7.1.0"
 
 //----------------------------------------------------------------------------------------
 // Configuration file (pluckerrc / plucker.ini)
@@ -373,14 +373,14 @@ enum {
     The L is required at end to let the wxConfig::Read() and Write() know it is a long
     property.
  */
-#define plkrIMAGES_ONLY_THUMBNAIL           0L
+#define plkrIMAGES_ONLY_THUMBNAIL           -1L
 
 //! Image handling alt_maxwidth/alt_maxheight. Tells to always link to the full-size one.
 /*!
     The L is required at end to let the wxConfig::Read() and Write() know it is a long
     property.
  */ 
-#define plkrIMAGES_ALWAYS_LINK              1000000L
+#define plkrIMAGES_ALWAYS_LINK              0L
 
 //! Image handling: default maxheight before an image is thumbnailed on a normal viewer.
 /*!
