@@ -1,5 +1,5 @@
 /*
- * $Id: bookmarkform.c,v 1.32.4.1 2003/12/07 10:18:48 nordstrom Exp $
+ * $Id: bookmarkform.c,v 1.37 2004/03/07 14:12:39 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -67,23 +67,22 @@ static UInt16       recordID  = NO_RECORD;
 static void AddBookmarkFormInit( void )
 {
     FormType*       addBookmarkForm;
-    Header*         record;
     Int16           length;
-    Char            name[100];
+    Char            name[ MAX_BOOKMARK_LEN + 1 ];
     Int16           i;
 
     addBookmarkForm = FrmGetFormPtr( frmAddBookmark );
     HanderaConvertForm( addBookmarkForm );
     fldPtr          = GetObjectPtr( frmAddBookmarkName );
 
-    record = MemHandleLock( GetCurrentRecord() );
-    TextifyRecord( record, name, 100 );
-    MemHandleUnlock( GetCurrentRecord() );
+    TextifyRecord( name, MAX_BOOKMARK_LEN );
     length = StrLen( name );
     /*eliminate raw control chars*/
-    for ( i=0; i< length; i++ )
-        if ( TxtGlueCharIsCntrl( name[ i ] ) )
+    for ( i = 0; i < length; i++ ) {
+        if ( TxtGlueCharIsCntrl( name[ i ] ) ) {
             name[ i ] = ' ';
+        }
+    }
     
     FldInsert( fldPtr, name, length );
     FldSetSelection( fldPtr, 0, length );
@@ -182,9 +181,12 @@ Boolean BookmarksFormHandleEvent
     EventType* event  /* pointer to an EventType structure */
     )
 {
-    Boolean handled = false;
+    UInt16  extEntries = 0;
+    Int16   extIndex   = NULL;
+    Boolean handled    = false;
 
-    recordID = NO_RECORD;
+    recordID   = NO_RECORD;
+    extEntries = CountExtBookmarks();
 
     switch ( event->eType ) {
         case frmOpenEvent:
@@ -204,7 +206,9 @@ Boolean BookmarksFormHandleEvent
 
                 list        = GetObjectPtr( frmBookmarksList );
                 selection   = LstGetSelection( list );
-                if ( selection != noListSelection ) {
+                if ( extEntries <= selection &&
+                     selection != noListSelection ) {
+
                     UInt16 choice;
 
                     choice = FrmCustomAlert( confirmDelete,
@@ -212,7 +216,7 @@ Boolean BookmarksFormHandleEvent
                                     NULL );
                     if ( choice == DELETE_BOOKMARK ) {
                         ErrTry {
-                            DeleteBookmark( selection );
+                            DeleteBookmark( selection - extEntries );
                             ReleaseBookmarkList();
                             CreateBookmarkList( list );
                             LstSetHeight( list, MAX_BOOKMARK_LIST_LEN );
@@ -235,18 +239,26 @@ Boolean BookmarksFormHandleEvent
                 list        = GetObjectPtr( frmBookmarksList );
                 selection   = LstGetSelection( list );
                 if ( selection != noListSelection ) {
-                    recordID = RestoreBookmarkData( selection + 2 );
+                    if ( selection >= extEntries )
+                        recordID = RestoreBookmarkData( selection -
+                            extEntries );
+                    else
+                        extIndex = selection;
                 }
             }
-            else if ( event->data.ctlEnter.controlID != frmBookmarksDone )
+            else if ( event->data.ctlEnter.controlID != frmBookmarksDone ) {
+                extEntries = 0;
                 break;
+            }
 
             ReleaseBookmarkList();
             FrmReturnToForm( PREVIOUS_FORM );
 
-            if ( recordID != NO_RECORD ) {
+            if ( recordID != NO_RECORD )
                 FrmUpdateForm( GetMainFormId(), frmViewRecord );
-            }
+            else if ( extIndex != NULL )
+                GotoExtBookmark( extIndex );
+
             handled = true;
             break;
 
