@@ -1,5 +1,5 @@
 /*
- * $Id: const.h,v 1.46.4.1 2003/10/24 02:43:31 prussar Exp $
+ * $Id: const.h,v 1.52 2004/02/28 15:28:21 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -68,9 +68,21 @@
 /* Type for keyboard map db */
 #define PlkrKeyboardMapType         'Keyb'
 
+/* Type for keyboard map db */
+#define PlkrVFSFontCacheType        'VFnC'
+
+/* Type for transliteration database */
+#define XlitDBType                  'Xlit'
+
 /* Type for armlet resources */
 #define ArmletResourceType          'armc'
 
+/* Type for I-mode icon db */
+#define PlkrImodeType               'Imod'
+
+/* Type for internal MemoDB */
+#define PlkrMemoDBType              'Memo'
+
 /* Length of OwnerID buffer */
 #define OWNER_ID_HASH_LEN           40
 
