@@ -1,5 +1,5 @@
 /*
- * $Id: control.c,v 1.112.4.1 2003/10/31 03:54:30 prussar Exp $
+ * $Id: control.c,v 1.128 2004/03/07 14:12:39 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -93,7 +93,7 @@ typedef struct {
 
 
 
-typedef Int16 ( * AnchorFinder )( void );
+typedef Int16 ( *AnchorFinder )( void );
 
 
 /***********************************************************************
@@ -101,18 +101,24 @@ typedef Int16 ( * AnchorFinder )( void );
  *      Local functions
  *
  ***********************************************************************/
-static void ClearPenData(void) CONTROL_SECTION;
-static void HighlightGraphicalControl(const Int16 controlID,
-                const AnchorStateType state) CONTROL_SECTION;
-static void HighlightControl(const Int16 controlID,
-                const AnchorStateType state) CONTROL_SECTION;
-static Int16 FindGraphicalControl(const Coord x, const Coord y) CONTROL_SECTION;
-static Int16 FindControlObject(const Coord x, const Coord y) CONTROL_SECTION;
-static void DoScreenAction(RegionType region) CONTROL_SECTION;
-static void HandleNextControl(void) CONTROL_SECTION;
-static void HandlePrevControl(void) CONTROL_SECTION;
-static void GoToFirstAnchor(void) CONTROL_SECTION;
-static void GoToLastAnchor(void) CONTROL_SECTION;
+static void ClearPenData( void ) CONTROL_SECTION;
+static void HighlightGraphicalControl( const Int16 controlID,
+                const AnchorStateType state ) CONTROL_SECTION;
+static void HighlightControl( const Int16 controlID,
+                const AnchorStateType state ) CONTROL_SECTION;
+static Int16 FindGraphicalControl( const Coord x, const Coord y ) CONTROL_SECTION;
+static Int16 FindControlObject( const Coord x, const Coord y ) CONTROL_SECTION;
+static void DoScreenAction( RegionType region ) CONTROL_SECTION;
+static void HandleNextControl( void ) CONTROL_SECTION;
+static void HandlePrevControl( void ) CONTROL_SECTION;
+static void GoToFirstAnchor( void ) CONTROL_SECTION;
+static void GoToLastAnchor( void ) CONTROL_SECTION;
+#ifdef SUPPORT_WORD_LOOKUP
+static Int16 FindSelectedWord( const Coord x, const Coord y ) CONTROL_SECTION;
+static void ShowSelectWordTapIcon(void) CONTROL_SECTION;
+static void HighlightSelectedWord( const AnchorStateType state ) CONTROL_SECTION;
+static Boolean InSelectedWord ( const Coord x, const Coord y ) CONTROL_SECTION;
+#endif
 
 
 /***********************************************************************
@@ -120,8 +126,66 @@ static void GoToLastAnchor(void) CONTROL_SECTION;
  *      Private variables
  *
  ***********************************************************************/
-static ControlDataType controls[NUMBER_OF_TOOLBAR_OBJECTS];
+
+static ControlDataType controls[ NUMBER_OF_TOOLBAR_OBJECTS ];
 static Int16 actualAnchor = 0;
+#ifdef SUPPORT_WORD_LOOKUP
+static ControlDataType  selectedWord;
+static Boolean          isSelectWordTapMode = false;
+#endif
+
+
+
+#ifdef SUPPORT_WORD_LOOKUP
+/* Indicate that the next tap looks things up in the selected word */
+static void ShowSelectWordTapIcon( void )
+{
+    FormType* mainForm;
+    UInt16    prevCoordSys;
+
+    if ( Prefs()->toolbar == TOOLBAR_NONE )
+        return;
+
+    mainForm = FrmGetFormPtr( GetMainFormId() );
+    prevCoordSys = PalmSetCoordinateSystem( STANDARD );
+
+    if ( Prefs()->toolbar == TOOLBAR_SILK ) {
+        /* FIXME: figure this out */
+    }
+    else {
+        FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpWait ) );
+
+        if ( isSelectWordTapMode ) {
+            FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpHome ) );
+            FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, bmpLookup ) );
+        }
+        else {
+            FrmHideObject( mainForm, FrmGetObjectIndex( mainForm, bmpLookup ) );
+            FrmShowObject( mainForm, FrmGetObjectIndex( mainForm, bmpHome ) );
+        }
+    }
+    PalmSetCoordinateSystem( prevCoordSys );
+}
+#endif
+
+
+
+/* Reset the prev/next anchor */
+void ResetActualAnchor( void )
+{
+    actualAnchor = 0;
+}
+
+
+
+/* Unhighlight and reset the anchor for the prev/next controls */
+void UnselectActualAnchor( void )
+{
+    if ( 0 < actualAnchor ) {
+        HighlightAnchor( actualAnchor - 1, ANCHOR_UNSELECTED );
+        actualAnchor = 0;
+    }
+}
 
 
 
@@ -232,7 +296,7 @@ static void ClearPenData( void )
 static void HighlightGraphicalControl
     (
     const Int16             controlID,  /* control ID of the object */
-    const AnchorStateType   state       /* anchor state ( ANCHOR_SELECTED or 
+    const AnchorStateType   state       /* anchor state ( ANCHOR_SELECTED or
                                            ANCHOR_UNSELECTED ) */
     )
 {
@@ -254,24 +318,49 @@ static void HighlightGraphicalControl
 
 
 
+#ifdef SUPPORT_WORD_LOOKUP
+/* Set highlight status for a graphical control object */
+static void HighlightSelectedWord
+    (
+    const AnchorStateType state       /* anchor state ( ANCHOR_SELECTED or
+                                         ANCHOR_UNSELECTED ) */
+    )
+{
+    if ( selectedWord.state != state ) {
+        HighlightRectangle( &selectedWord.bounds, 0,
+            ( state == ANCHOR_SELECTED ) ? true : false, BUTTON );
+    }
+}
+#endif
+
+
+
 /* Determine whether an anchor or control object should change highlight
    status */
 static void HighlightControl
     (
     const Int16             controlID,  /* control ID of the object */
-    const AnchorStateType   state       /* anchor state ( ANCHOR_SELECTED or 
+    const AnchorStateType   state       /* anchor state ( ANCHOR_SELECTED or
                                            ANCHOR_UNSELECTED ) */
     )
 {
     if ( 0 < controlID )
         HighlightAnchor( controlID - 1, state );
-    else if ( controlID < 0 )
-        HighlightGraphicalControl( -( controlID + 1 ), state );
+    else if ( controlID < 0 ) {
+#ifdef SUPPORT_WORD_LOOKUP
+        if ( controlID == - ( SELECTEDWORDCONTROL + 1 ) )
+        {
+            HighlightSelectedWord( state );
+            return;
+        }
+#endif
+        HighlightGraphicalControl( - ( controlID + 1 ), state );
+    }
 }
 
 
 
-/* Find control object at given location, return control ID, or NOT_FOUND 
+/* Find control object at given location, return control ID, or NOT_FOUND
    if no object was found at the given location */
 static Int16 FindGraphicalControl
     (
@@ -281,7 +370,7 @@ static Int16 FindGraphicalControl
 {
     Int16 controlID;
 
-    for ( controlID = 0; controlID < NUMBER_OF_TOOLBAR_OBJECTS; controlID++ )
+    for ( controlID = 0 ; controlID < NUMBER_OF_TOOLBAR_OBJECTS ; controlID++ )
         if ( RctPtInRectangle( x, y, &controls[ controlID ].bounds ) )
             return controlID;
 
@@ -321,6 +410,43 @@ static Int16 FindControlObject
 
 
 
+#ifdef SUPPORT_WORD_LOOKUP
+/* Is tap in last selected word? */
+static Boolean InSelectedWord
+    (
+    const Coord x,
+    const Coord y
+    )
+{
+    return RctPtInRectangle( x, y, &selectedWord.bounds );
+}
+
+
+
+/* Find control object at pen location */
+static Int16 FindSelectedWord
+    (
+    const Coord x,
+    const Coord y
+    )
+{
+    /* event occured in the toolbar */
+    if ( y <= TopLeftY() || TopLeftY() + ExtentY() <= y )
+        return 0;
+
+    ShowSelectWordTapIcon();
+    DoFindSelectedWord( x, y );
+    if ( GetSelectedWord( &selectedWord.bounds ) != NULL ) {
+        return -( SELECTEDWORDCONTROL + 1 );
+    }
+
+    /* event did not occur on anything */
+    return 0;
+}
+#endif
+
+
+
 /* Perform action for specified screen region */
 static void DoScreenAction
     (
@@ -335,17 +461,19 @@ static void DoScreenAction
 /* Jump to requested record */
 void JumpToRecord
     (
-    const UInt16  record, /* record ID */
-    const Int16   pOffset, /* offset to first paragraph */
-    const Int16   cOffset  /* offset to first character */
+    const UInt16  recordId, /* record ID */
+    const Int16   pOffset,  /* offset to first paragraph */
+    const Int16   cOffset   /* offset to first character */
     )
 {
     ErrTry {
-        ViewRecord( record, true, pOffset, cOffset );
-        SetVisitedLink( record );
+        ViewRecord( recordId, ADD_TO_HISTORY, pOffset, cOffset,
+            WRITEMODE_DRAW_CHAR );
+        SetVisitedLink( recordId );
     }
     ErrCatch( UNUSED_PARAM( err ) ) {
-        ViewRecord( GetHistoryPrev(), false, NO_OFFSET, NO_OFFSET );
+        ViewRecord( GetHistoryPrev(), FROM_HISTORY, NO_OFFSET, NO_OFFSET,
+            WRITEMODE_DRAW_CHAR );
     } ErrEndCatch
 }
 
@@ -365,25 +493,24 @@ void DoControlAction
     const Int16 control /* control value of the object */
     )
 {
-    UInt16 newRecord;
-    Int16 controlID;
+    UInt16  newRecord;
+    Int16   controlID;
 
     /* Clear location of find pattern */
     ClearFindPatternData();
 
     if ( 0 < control ) {
-        Int16 anchorIndex;
-        UInt16 reference;
-        Int16 offset;
-        UInt16 image;
+        Int16   anchorIndex;
+        UInt16  reference;
+        Int16   offset;
+        UInt16  image;
 
         anchorIndex = control - 1;
         reference   = GetVisibleReference( anchorIndex );
         offset      = GetVisibleOffset( anchorIndex );
         image       = GetVisibleImage( anchorIndex );
 
-        if ( Prefs()->linkClick )
-            SndPlaySystemSound( sndClick );
+        SndPlaySystemSound( sndClick );
 
         if ( image != 0 ) {
             RectangleType   bounds;
@@ -406,15 +533,18 @@ void DoControlAction
             y += bounds.topLeft.y;
             LstSetPosition( list, x, y );
 
-            prevCoordSys = PalmSetCoordinateSystem( STANDARD );
-            selection = LstPopupList( list );
+            prevCoordSys    = PalmSetCoordinateSystem( STANDARD );
+            selection       = LstPopupList( list );
             PalmSetCoordinateSystem( prevCoordSys );
-            if ( selection == noListSelection )
+            if ( selection == noListSelection ) {
                 return;
-            else if ( selection == 1 )
+            }
+            else if ( selection == 1 ) {
                 reference = image;
+            }
         }
         JumpToRecord( reference, offset, NO_OFFSET );
+
         return;
     }
 
@@ -425,7 +555,8 @@ void DoControlAction
     switch ( controlID ) {
         case HOMECONTROL:
             DoAutoscrollToggle( AUTOSCROLL_OFF );
-            ViewRecord( HOME_PAGE_ID, true, NO_OFFSET, NO_OFFSET );
+            ViewRecord( HOME_PAGE_ID, ADD_TO_HISTORY, NO_OFFSET, NO_OFFSET,
+                WRITEMODE_DRAW_CHAR );
             SetVisitedLink( HOME_PAGE_ID );
             break;
 
@@ -433,7 +564,8 @@ void DoControlAction
             DoAutoscrollToggle( AUTOSCROLL_OFF );
             newRecord = GetHistoryPrev();
             if ( newRecord != NO_RECORD ) {
-                ViewRecord( newRecord, false, NO_OFFSET, NO_OFFSET );
+                ViewRecord( newRecord, FROM_HISTORY, NO_OFFSET, NO_OFFSET,
+                    WRITEMODE_DRAW_CHAR );
                 SetVisitedLink( newRecord );
             }
             else {
@@ -445,7 +577,8 @@ void DoControlAction
             DoAutoscrollToggle( AUTOSCROLL_OFF );
             newRecord = GetHistoryNext();
             if ( newRecord != NO_RECORD ) {
-                ViewRecord( newRecord, false, NO_OFFSET, NO_OFFSET );
+                ViewRecord( newRecord, FROM_HISTORY, NO_OFFSET, NO_OFFSET,
+                    WRITEMODE_DRAW_CHAR );
                 SetVisitedLink( newRecord );
             }
             break;
@@ -491,6 +624,7 @@ void DoControlAction
             UInt16      prevCoordSys;
             ListType*   list;
             Int16       selection;
+            Int16       extEntries;
             UInt16      numOfBookmarks;
 
             DoAutoscrollToggle( AUTOSCROLL_OFF );
@@ -515,35 +649,51 @@ void DoControlAction
                 FrmPopupForm( frmBookmarks );
             }
             else if ( selection != noListSelection ) {
-                newRecord = RestoreBookmarkData( selection );
-                if ( newRecord != NO_RECORD ) {
-                    ViewRecord( newRecord, false, NO_OFFSET, NO_OFFSET );
-                    SetVisitedLink( newRecord );
+                selection -= 2;
+                extEntries = CountExtBookmarks();
+                if ( selection < extEntries )
+                    GotoExtBookmark( selection );
+                else {
+                    selection -= extEntries;
+                    newRecord = RestoreBookmarkData( selection );
+                    if ( newRecord != NO_RECORD ) {
+                        ViewRecord( newRecord, FROM_HISTORY, NO_OFFSET, NO_OFFSET,
+                            WRITEMODE_DRAW_CHAR );
+                        SetVisitedLink( newRecord );
+                    }
                 }
             }
             break;
-        }        
-                    
-        case AUTOSCROLLSTARTCONTROL: {
-        case AUTOSCROLLSTOPCONTROL: 
-            DoAutoscrollToggle( AUTOSCROLL_TOGGLE );
-            break;
         }
 
-        case AUTOSCROLLINCRCONTROL: {
+        case AUTOSCROLLSTARTCONTROL:
+        case AUTOSCROLLSTOPCONTROL:
+            DoAutoscrollToggle( AUTOSCROLL_TOGGLE );
+            break;
+
+        case AUTOSCROLLINCRCONTROL:
             DoAutoscrollIncr();
-            break;    
-        }
-        
-        case AUTOSCROLLDECRCONTROL: {
+            break;
+
+        case AUTOSCROLLDECRCONTROL:
             DoAutoscrollDecr();
             break;
-        }
-    
-        case COPYTOMEMOCONTROL: {
+
+        case COPYTOMEMOCONTROL:
             DoHardcopy();
             break;
+
+#ifdef SUPPORT_WORD_LOOKUP
+        case SELECTEDWORDCONTROL: {
+            EventType newEvent;
+
+            MemSet( &newEvent, sizeof( newEvent ), 0 );
+            newEvent.eType = pluckerSelectedWordEvent;
+            EvtAddEventToQueue( &newEvent );
+
+            break;
         }
+#endif
 
         default:
             break;
@@ -555,12 +705,12 @@ void DoControlAction
 /* Emit a key */
 void EmitKey( UInt16 key, UInt16 modifiers )
 {
-  EventType newEvent;
-  MemSet(&newEvent, sizeof(newEvent), 0);
-  newEvent.eType = keyDownEvent;
-  newEvent.data.keyDown.chr = key;
-  newEvent.data.keyDown.modifiers = modifiers;
-  EvtAddEventToQueue(&newEvent);
+    EventType newEvent;
+    MemSet(&newEvent, sizeof(newEvent), 0);
+    newEvent.eType = keyDownEvent;
+    newEvent.data.keyDown.chr = key;
+    newEvent.data.keyDown.modifiers = modifiers;
+    EvtAddEventToQueue(&newEvent);
 }
 
 
@@ -588,6 +738,17 @@ void DoSelectTypeAction
         switch ( selection ) {
             case SELECT_NONE:
                 break;
+                
+#ifdef SUPPORT_WORD_LOOKUP
+            case SELECT_WORD_LOOKUP:
+                isSelectWordTapMode = ! isSelectWordTapMode;
+                ShowSelectWordTapIcon();
+                break;
+#endif
+
+            case SELECT_SAVE_POSITION:
+                AddToHistory( GetCurrentRecordId() );
+                break;
 
             case SELECT_BRIGHTNESS_ADJUST:
                 EmitKey( vchrBrightness, commandKeyMask );
@@ -845,9 +1006,11 @@ void DoSelectTypeAction
                     else
                         Prefs()->toolbar = Prefs()->savedToolbar;
                 }
+#ifdef HAVE_SILKSCREEN
                 if ( GetSilkScreenStatus() == SILKSCREEN_NONE &&
                    Prefs()->toolbar != TOOLBAR_NONE )
                     SetSilkScreenStatus( Prefs()->savedSilkscreen );
+#endif                    
                 FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
                 break;
 
@@ -868,18 +1031,22 @@ void DoSelectTypeAction
                         ( ! sonySilk ||
                           Prefs()->savedSilkscreen == SILKSCREEN_DOWN )
                       ) {
+#ifdef HAVE_SILKSCREEN
                         if ( sonySilk ) {
                             SetSilkScreenStatus( SILKSCREEN_DOWN );
                             setSilk = true;
                         }
+#endif
                         Prefs()->toolbar   = TOOLBAR_TOP;
                         Prefs()->scrollbar = SCROLLBAR_RIGHT;
                     }
                     else {
+#ifdef HAVE_SILKSCREEN
                         if ( sonySilk ) {
                             SetSilkScreenStatus( Prefs()->savedSilkscreen );
                             setSilk = true;
                         }
+#endif
                         Prefs()->toolbar   = Prefs()->savedToolbar;
                         Prefs()->scrollbar = Prefs()->savedScrollbar;
                     }
@@ -887,6 +1054,7 @@ void DoSelectTypeAction
                 else {
                     Prefs()->savedToolbar   = Prefs()->toolbar;
                     Prefs()->savedScrollbar = Prefs()->scrollbar;
+#ifdef HAVE_SILKSCREEN
                     if ( sonySilk ) {
                         Prefs()->savedSilkscreen = GetSilkScreenStatus();
                         if ( Prefs()->savedSilkscreen == SILKSCREEN_NONE )
@@ -894,12 +1062,18 @@ void DoSelectTypeAction
                         SetSilkScreenStatus( SILKSCREEN_NONE );
                         setSilk = true;
                     }
+#endif
                     Prefs()->toolbar        = TOOLBAR_NONE;
                     Prefs()->scrollbar      = SCROLLBAR_NONE;
                 }
-                if ( setSilk )
+#ifdef HAVE_SILKSCREEN
+                if ( setSilk ) {
                     FrmGotoForm( GetMainFormId() );
+                    Prefs()->hideSonyStatusBar = ( GetSilkScreenStatus() 
+                        == SILKSCREEN_NONE );
+                }
                 else
+#endif
                     FrmUpdateForm( Prefs()->lastForm, frmRedrawUpdateCode );
                 break;
             }
@@ -924,18 +1098,39 @@ Boolean HandlePenDown
     PenData.penDownX = PenData.moveFromX = x;
     PenData.penDownY = PenData.moveFromY = y;
 
+#ifdef HAVE_SILKSCREEN
     if ( SILKSCREEN_NONE == GetSilkScreenStatus() &&
         SILKSCREEN_HEIGHT_HIRES_SONY + SONY_SCREEN_HEIGHT <= y ) {
         if ( Prefs()->toolbar == TOOLBAR_NONE &&
-           Prefs()->scrollbar == SCROLLBAR_NONE )
+           Prefs()->scrollbar == SCROLLBAR_NONE ) {
             DoSelectTypeAction( SELECT_TOGGLE_FULLSCREEN );
-        else
+        }
+        else {
+            if ( Prefs()->savedSilkscreen == SILKSCREEN_NONE ) {
+                Prefs()->savedSilkscreen = SILKSCREEN_DOWN;
+            }
             SetSilkScreenStatus( Prefs()->savedSilkscreen );
+            Prefs()->hideSonyStatusBar = false;
+        }
         handled = true;
         return handled;
     }
+#endif
 
-    PenData.selectedControl = FindControlObject( x, y );
+#ifdef SUPPORT_WORD_LOOKUP
+    if ( isSelectWordTapMode ) {
+        PenData.selectedControl = FindSelectedWord( x, y );
+        isSelectWordTapMode     = false;
+        ShowSelectWordTapIcon();
+    }
+    else if ( Prefs()->selectWordTap == SELECT_WORD_TAP_ONCE ) {
+        PenData.selectedControl = FindControlObject( x, y );
+        if ( PenData.selectedControl == 0 )
+            PenData.selectedControl = FindSelectedWord( x, y );
+    }
+    else
+#endif
+        PenData.selectedControl = FindControlObject( x, y );
 
     if ( PenData.selectedControl != 0 ) {
         HighlightControl( PenData.selectedControl, ANCHOR_SELECTED );
@@ -946,7 +1141,7 @@ Boolean HandlePenDown
         RegionType region;
         Coord      rotX;
         Coord      rotY;
-        
+
         rotX = x;
         rotY = y;
 
@@ -1040,14 +1235,19 @@ Boolean HandlePenUp
     if ( PenData.selectedControl != 0 )
         HighlightControl( PenData.selectedControl, ANCHOR_UNSELECTED );
 
-    if ( PenData.selectedControl == FindControlObject( x, y ) )
+    if (
+#ifdef SUPPORT_WORD_LOOKUP
+          ( PenData.selectedControl == -( SELECTEDWORDCONTROL + 1 ) &&
+            InSelectedWord( x, y ) ) ||
+#endif
+         PenData.selectedControl == FindControlObject( x, y ) ) {
         DoControlAction( PenData.selectedControl );
+    }
 
     ClearPenData();
     PenData.moveFromX = -1;
     PenData.moveFromY = -1;
 
-
     return true; /* always handled */
 }
 
@@ -1082,7 +1282,7 @@ void GotoLocation
                 location * ( GetSequenceHeight() / 10 + 1 );
 
     if ( NO_RECORD == GetSequentialRecordId( DIRECTION_DOWN ) &&
-         GetSequenceOffset() + ExtentY() - delta >= GetSequenceHeight() )
+         GetSequenceHeight() <= GetSequenceOffset() + ExtentY() - delta )
         /* delta would put us too far down */
         delta = GetSequenceOffset() + ExtentY() - GetSequenceHeight();
 
@@ -1099,9 +1299,9 @@ void DoAutoscrollToggle
 {
     FormType* mainForm;
     UInt16    prevCoordSys;
-    
-    if ( toggle == AUTOSCROLL_OFF && 
-         ! Prefs()->autoscrollEnabled ) 
+
+    if ( toggle == AUTOSCROLL_OFF &&
+         ! Prefs()->autoscrollEnabled )
         return;
 
     mainForm = FrmGetFormPtr( GetMainFormId() );
@@ -1212,3 +1412,4 @@ Boolean IsMainFormWinActive( void )
     else
         return false;
 }
+
