@@ -1,5 +1,5 @@
 /*
- * $Id: fontform.c,v 1.30.4.1 2003/11/07 21:17:06 prussar Exp $
+ * $Id: fontform.c,v 1.36 2004/02/01 11:26:33 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -210,7 +210,6 @@ static void FontFormInit( void )
     UInt16          i;
     ControlType*    ctl;
     ListType*       list;
-    Char            formTitle[ 30 ];
 
     if ( IsFormMain( Prefs()->lastForm ) ) {
         cameFromLibrary   = false;
@@ -236,6 +235,10 @@ static void FontFormInit( void )
 #endif
     else
         buttonCount = BUTTON_COUNT_OS2;
+        
+#ifdef SUPPORT_VFS_FONTS
+    ScanVFSFonts();
+#endif
 
     form        = FrmGetFormPtr( fontFormID );
     buttonList  = SafeMemPtrNew( buttonCount * sizeof *buttonList );
@@ -377,10 +380,6 @@ static void FontFormInit( void )
 
     FrmDrawForm( form );
 
-    /* Move the title to the right, to make room for the help icon */
-    StrPrintF( formTitle, "      %s", FrmGetTitle( form ) );
-    FrmSetTitle( form, formTitle );
-
     /* Highlight the pushbutton that reflects the currently set font */
     for ( i = 0; i < buttonCount; i++ ) {
        if ( fontModePref == buttonList[ i ].fontMode ) {
@@ -390,7 +389,8 @@ static void FontFormInit( void )
     }
 
     if ( userFontButtonNumber != NO_USER_FONT_BUTTON ) {
-        userFontSelection = GetUserFontNumber( prefsUserFontName );
+        userFontSelection = GetUserFontNumber( prefsUserFontName, true, 
+                                fontCacheOff );
         if ( userFontSelection == NO_SUCH_USER_FONT )
             userFontSelection = 0;
         list        = GetObjectPtr( frmFontUserFontList );
@@ -447,8 +447,10 @@ void FontDoOK( void )
     updateDoc    = false;
     lineSpacing += MINIMAL_LINE_SPACING;
 
-    if ( userFontButtonNumber != NO_USER_FONT_BUTTON && GetNumberOfUserFonts() != 0 &&
-        userFontSelection != GetUserFontNumber( prefsUserFontName )
+    if ( userFontButtonNumber != NO_USER_FONT_BUTTON && 
+        GetNumberOfUserFonts() != 0 &&
+        userFontSelection != GetUserFontNumber( prefsUserFontName, true, 
+                                 fontCacheOff )
         )
         changeUserFont = true;
     else
@@ -464,8 +466,9 @@ void FontDoOK( void )
         ) {
         if ( ! IsFullscreenformActive() ) {
             if ( ! cameFromLibrary ) {
-                if ( Prefs()->individualDocumentFonts )
+                if ( Prefs()->individualDocumentFonts ) {
                     GetHistoryPtr()->font = buttonList[ selected ].fontMode;
+                }
                 else {
                     Prefs()->fontModeMain = buttonList[ selected ].fontMode;
                     Prefs()->defaultFontModeMain = Prefs()->fontModeMain;
@@ -525,8 +528,7 @@ void FontDoOK( void )
         FrmReturnToForm( frmFullscreen );
     } else {
         if ( ! cameFromLibrary && Prefs()->individualDocumentFonts &&
-            CtlGetValue( GetObjectPtr( frmFontAsDefault ) )
-          ) {
+             CtlGetValue( GetObjectPtr( frmFontAsDefault ) ) ) {
             Prefs()->defaultFontModeMain     = GetHistoryPtr()->font;
             Prefs()->defaultLineSpacing      = GetHistoryPtr()->lineSpacing;
             Prefs()->defaultParagraphSpacing =
@@ -625,6 +627,7 @@ Boolean FontFormHandleEvent
             }
             break;
         }
+
         case ctlSelectEvent:
             switch ( event->data.ctlEnter.controlID ) {
                 case frmFont1:
@@ -672,6 +675,7 @@ Boolean FontFormHandleEvent
                     break;
             }
             break;
+
         case frmOpenEvent:
             FontFormInit();
             handled = true;
