@@ -1,5 +1,5 @@
 /*
- * $Id: fullscreenform.c,v 1.27.4.5 2003/11/14 16:52:23 prussar Exp $
+ * $Id: fullscreenform.c,v 1.44 2004/03/07 11:17:25 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -87,7 +87,7 @@ static Int16         tableY;
 static Boolean       penDownInFrame;
 static Boolean       fullscreenformActive = false;
 static RectangleType fullBounds;
-static Boolean       fastEnough;	/* SuperVZ or better */
+static Boolean       fastEnough;    /* SuperVZ or better */
 static Coord         originalBackButtonHeight = 0;
 static Coord         originalBackButtonWidth = 0;
 
@@ -99,8 +99,8 @@ static Coord         originalBackButtonWidth = 0;
  ***********************************************************************/
 static Boolean FsFormInit( void ) FULLSCREENFORM_SECTION;
 static void MoveImage( Coord x, Coord y ) FULLSCREENFORM_SECTION;
-static void DrawImage( ImageType* image, Coord drawX, Coord drawY,
-    RectangleType* bounds ) FULLSCREENFORM_SECTION;
+static void DrawImage( ImageType* image, Coord drawX, Coord drawY, 
+                RectangleType* bounds ) FULLSCREENFORM_SECTION;
 static void DrawScanner( void ) FULLSCREENFORM_SECTION;
 static void DrawButton( void ) FULLSCREENFORM_SECTION;
 static void FreeMemory( void ) FULLSCREENFORM_SECTION;
@@ -110,7 +110,7 @@ static Boolean HandleFsKeyDown(const EventType* event) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenUp( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenMove( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
 static Boolean HandleFsPenDown( const Coord x, const Coord y) FULLSCREENFORM_SECTION;
-static Boolean FsHandleMenuEvent ( UInt16 itemID ) FULLSCREENFORM_SECTION; 
+static Boolean FsHandleMenuEvent ( UInt16 itemID ) FULLSCREENFORM_SECTION;
 
 
 /* Initialize the image form */
@@ -124,7 +124,7 @@ static Boolean FsFormInit( void )
     RectangleType   newBounds;
 
     err = FtrGet( sysFileCSystem, sysFtrNumProcessorID, &processorType );
-    if ( err == errNone && processorType >= 0x00040000 )
+    if ( err == errNone && 0x00040000 <= processorType )
         fastEnough = true;
     else
         fastEnough = false;
@@ -141,6 +141,12 @@ static Boolean FsFormInit( void )
 
     fsForm = FrmGetFormPtr( GetValidForm( frmFullscreen ) );
 
+    backButtonIndex = FrmGetObjectIndex( fsForm, frmFullscreenBack );
+
+    FrmHideObject( fsForm, backButtonIndex );
+
+    FrmDrawForm( fsForm );
+
     screenWindow = WinGetDrawWindow();
 
     screenBounds.topLeft.x = 0;
@@ -153,12 +159,6 @@ static Boolean FsFormInit( void )
     WinSetBounds( screenWindow, &newBounds );
     HiResAdjustBounds( &screenBounds, palmHiRes );
 
-    backButtonIndex = FrmGetObjectIndex( fsForm, frmFullscreenBack );
-
-    FrmHideObject( fsForm, backButtonIndex );
-
-    FrmDrawForm( fsForm );
-
     bufferWindow = NULL;
     if ( Support35() ) { /* 3.0 (at least) mis-behaves terribly */
         bufferWindow = WinCreateOffscreenWindow( screenBounds.extent.x,
@@ -193,7 +193,7 @@ static Boolean FsFormInit( void )
     frameBounds.topLeft.y = RotMaxExtentY() - frameBounds.extent.y - 1;
 
     /* adjust position of back button for displays taller than 160 */
-    if ( 0 == originalBackButtonHeight ) {
+    if ( originalBackButtonHeight == 0 ) {
         FrmGetObjectBounds( fsForm, backButtonIndex, &ctlBounds );
         HiResAdjustBounds( &ctlBounds, sonyHiRes );
         originalBackButtonWidth  = ctlBounds.extent.x;
@@ -203,18 +203,21 @@ static Boolean FsFormInit( void )
     ctlBounds.topLeft.y = RotMaxExtentY() - originalBackButtonHeight;
     ctlBounds.extent.x  = originalBackButtonWidth;
     ctlBounds.extent.y  = originalBackButtonHeight;
-
+    
     if ( fastEnough ) {
-        ControlType*    backButton;
+        ControlType* backButton;
+
         backButton = GetObjectPtr( frmFullscreenBack );
         CtlGlueSetFont( backButton, symbolFont );
         switch ( Prefs()->rotate ) {
             case ROTATE_PLUS90:
                 CtlSetLabel( backButton, "\006" );
                 break;
+
             case ROTATE_MINUS90:
                 CtlSetLabel( backButton, "\005" );
                 break;
+
             case ROTATE_ZERO:
             default:
                 CtlSetLabel( backButton, "\003" );
@@ -272,19 +275,22 @@ void FsAdjustVerticalOffset
     Int16   adjustment  /* adjustment in pixels */
     )
 {
-    imageBounds.topLeft.y -= adjustment;
+    Coord x;
+    Coord y;
 
-    if ( ( imageBounds.extent.y - MaxExtentY() ) <= imageBounds.topLeft.y )
-        imageBounds.topLeft.y = imageBounds.extent.y - MaxExtentY();
-    if ( imageBounds.topLeft.y < 0 )
-        imageBounds.topLeft.y = 0;
+    x = screenBounds.topLeft.x;
+    y = screenBounds.topLeft.y - adjustment;
 
-    DrawImage( image, -imageBounds.topLeft.x, -imageBounds.topLeft.y,
-               &fullBounds );
+    y = min( imageBounds.extent.y - screenBounds.extent.y, y );
+    y = max( 0, y );
+
+    imageBounds.topLeft.x = screenBounds.topLeft.x = x;
+    imageBounds.topLeft.y = screenBounds.topLeft.y = y;
+
+    DrawImage( image, -x, -y, &fullBounds );
 
     if ( IsLargeTable() )
-        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
-                          imageBounds.topLeft.y );
+        CopyTableAnchors( image->reference, x, y );
 }
 
 
@@ -309,11 +315,8 @@ static void MoveImage
         WinCopyRectangle( bufferWindow, screenWindow, &bufferBounds, 0, 0,
             winPaint );
     }
-
-    if ( Prefs()->visualAid ) {
-        DrawButton();
-        DrawScanner();
-    }
+    DrawButton();
+    DrawScanner();
 }
 
 
@@ -398,7 +401,7 @@ static void DrawImage
                 }
             }
         }
-        /* Specifically erase any whitespace beyound the image's bounds */
+        /* Specifically erase any whitespace beyond the image's bounds */
         if ( imageBounds.extent.x < screenBounds.extent.x ) {
             whitespace.topLeft.y = 0;
             whitespace.topLeft.x = imageBounds.extent.x;
@@ -480,7 +483,7 @@ Boolean FsDoSelectTypeAction
 
     handled = false;
 
-    switch( selection ) {
+    switch ( selection ) {
         case SELECT_GO_FORWARD:
             /* Currently you cannot go forward to anything from here */
             handled = true;
@@ -492,6 +495,7 @@ Boolean FsDoSelectTypeAction
             FrmReturnToForm( PREVIOUS_FORM );
             /* Not handled because we do want to run their normal functions
                from within DoSelectTypeAction() */
+            handled = false;
             break;
 
         case SELECT_GO_TO_TOP:
@@ -655,6 +659,7 @@ static Boolean HandleFsKeyDown(const EventType* event)
                 DoPageMove( -RotGetScrollValue() );
             handled = true;
             break;
+
 #ifdef HAVE_FIVEWAY_SDK
         case vchrNavChange:
             if ( NavKeyPressed( event, Left ) &&
@@ -677,6 +682,7 @@ static Boolean HandleFsKeyDown(const EventType* event)
             }
             break;
 #endif
+
         case spaceChr:
             if ( Prefs()->gestMode[ GESTURES_RIGHT ] != SELECT_NONE &&
                  Prefs()->gestures )
@@ -742,22 +748,23 @@ static Boolean HandleFsPenUp
     const Coord y
     )
 {
-    if ( IsLargeTable() ) {
-        if ( anchor != NOT_FOUND ) {
-            HighlightAnchor( anchor, ANCHOR_UNSELECTED );
-            if ( ( abs(x - tableX) < 4 ) && ( abs(y - tableY ) < 4 ) ) {
-                FsDoControlAction( anchor + 1 );
-                return true;
-            }
+    if ( IsLargeTable() && anchor != NOT_FOUND ) {
+        HighlightAnchor( anchor, ANCHOR_UNSELECTED );
+        if ( ( abs(x - tableX) < 4 ) && ( abs(y - tableY ) < 4 ) ) {
+            FsDoControlAction( anchor + 1 );
+            return true;
         }
-        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
-                          imageBounds.topLeft.y );
     }
+
     anchor = NOT_FOUND;
 
     DrawImage( image, -imageBounds.topLeft.x, -imageBounds.topLeft.y,
                &fullBounds );
 
+    if ( IsLargeTable() )
+        CopyTableAnchors( image->reference, imageBounds.topLeft.x,
+                          imageBounds.topLeft.y );
+
     penDownInFrame = false;
 
     return true;
@@ -876,10 +883,9 @@ static Boolean HandleFsPenDown
     if ( penDownInFrame && HandleFsPenMove( x, y ) )
         return true;
 
-    if ( Prefs()->visualAid ) {
-        DrawButton();
-        DrawScanner();
-    }
+    DrawButton();
+    DrawScanner();
+
     return true;
 }
 
@@ -911,7 +917,7 @@ Boolean FsFormHandleEvent
             break;
 
         case keyDownEvent:
-            handled = HandleFsKeyDown(event);
+            handled = HandleFsKeyDown( event );
             break;
 
         case penDownEvent:
@@ -949,7 +955,7 @@ Boolean FsFormHandleEvent
         case frmUpdateEvent:
             FreeMemory();
             menuActive = false;
-            if ( FsFormInit() == false ) {
+            if ( ! FsFormInit() ) {
                 /* Do something */
             }
             handled = true;
@@ -957,7 +963,7 @@ Boolean FsFormHandleEvent
 
         case frmOpenEvent:
             menuActive = false;
-            if ( FsFormInit() == false ) {
+            if ( ! FsFormInit() ) {
                 /* Do something */
             }
             handled = true;
@@ -991,8 +997,10 @@ Boolean FsFormHandleEvent
 
 static void FreeMemory( void )
 {
-    if ( bufferWindow != NULL )
+    if ( bufferWindow != NULL ) {
         WinDeleteWindow( bufferWindow, false );
+        bufferWindow = NULL;
+    }
 
     if ( image->type == MULTIIMAGE ) {
         MultiImageNodeType* node;
