@@ -1,5 +1,5 @@
 /*
- * $Id: grayfont.c,v 1.13.4.1 2003/11/13 15:08:45 prussar Exp $
+ * $Id: grayfont.c,v 1.37 2004/03/05 14:48:27 prussar Exp $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,20 +20,27 @@
 #include <PalmOS.h>
 #include <FntGlue.h>
 #include <TxtGlue.h>
-
+#include "const.h"
 #include "os.h"
 #include "util.h"
 #include "prefsdata.h"
 #include "palmbitmap.h"
+#include "font.h"
 #define NO_GRAY_FONT_SUBSTITUTION
 #include "grayfont.h"
 
+#ifdef HAVE_PALMCUNIT
+#include "mock_Header.h"
+#endif
+
 
 /***********************************************************************
  *
  *      Internal Constants
  *
  ***********************************************************************/
+#define KERNING_FONTVERSION                     2
+#define MAX_SCREEN_WIDTH                        500
 #define MAX_FONT_ID                             255
 #define HIGH_DENSITY_FEATURE_SET_VERSION        4
 #define RESOURCE_NAME_ID             'G'
@@ -42,6 +49,8 @@
 #define RESOURCE_NAME_BITMAP_VERSION 2
 #define RESOURCE_NAME_DEPTH          3
 
+#define REMAP_FONTVERSION            0x4000
+
 
 
 
@@ -50,6 +59,12 @@
  *      Internal types
  *
  ***********************************************************************/
+
+typedef struct {
+    UInt16 fontVersion;
+    UInt16 fontID;
+} RemapInfoType;
+
 typedef struct {
     UInt16 firstGlyph;   /* first glyph found in resource */
     UInt16 lastGlyph;    /* last glyph found in resource */
@@ -69,6 +84,9 @@ typedef struct {
     UInt16 numberOfBitmapResources;
     UInt16 bitmapResourceTableOffset;
     UInt16 glyphInfoTableOffset;
+    Int16  minLeftKerning;
+    Int16  maxRightOverhang;
+    UInt16 reserved[4];
 } GrayFontType;
 
 typedef struct {
@@ -85,10 +103,16 @@ typedef struct {
     UInt16  reserved;
 } GrayFontGlyphInfo;
 
+typedef struct {
+    MemHandle        handle;
+    FontResourceKind kind;
+} ResourceCacheEntry;    
+
 typedef struct {
     GrayFontType         header;
     GrayFontGlyphInfo*   glyphList;
     GrayFontBitmapsInfo* bitmapsResourceIndex;
+    ResourceCacheEntry*  resourceCache;
 } GrayFontInfo;
 
 typedef union {
@@ -106,13 +130,15 @@ typedef union {
 static void GrayFntDelete ( FontID font ) GRAYFONT_SECTION;
 static GrayFontGlyphInfo* GetGlyph ( WChar ch ) GRAYFONT_SECTION;
 static UInt16 TempBitmapSize ( UInt16 width, UInt16 height ) GRAYFONT_SECTION;
-static void ColorizeBitmapSetColorMap ( RGBColorType* color, Boolean invert )
+static void ColorizeBitmapSetColorMap ( RGBColorType* fore, 
+    RGBColorType* back, Boolean invert )
     GRAYFONT_SECTION;
-static void ColorizeBitmap4 ( PalmBitmapType* dest, PalmBitmapType* src,
-    Boolean scanLine ) GRAYFONT_SECTION;
-static void InvertBitmap4 ( PalmBitmapType* dest, PalmBitmapType* src,
-    Boolean scanLine )
+static void ColorizeBitmap4To16 ( UInt16 x, UInt16 y, PalmBitmapType* dest,
+    PalmBitmapType* src ) GRAYFONT_SECTION;
+static void InvertBitmap4 ( PalmBitmapType* dest, PalmBitmapType* src )
     GRAYFONT_SECTION;
+static void GrayFntClearCaches( void ) GRAYFONT_SECTION;
+
 
 
 
@@ -132,44 +158,70 @@ static Boolean         active = false;
 static PalmBitmapType* tempBitmap = NULL;
 static UInt16          tempBitmapSize = 0;
 static UInt16          tempBitmapHeaderSize;
-static UInt16          colorMap[ 16 ];
+static Boolean         fastRenderAvailable = false;
+static Boolean         backgroundErase     = true;
+static RGBColorType    lastFore = { 0, 0, 0 };
+static RGBColorType    lastBack = { 255, 255, 255 };
+static UInt16          colorMap[ 16 ] = {
+    0xffff, 0xef7d, 0xdefb, 0xce79, 0xbdd7, 0xad55, 0x9cd3, 0x8c51, 0x73ae,
+    0x632c, 0x52aa, 0x4228, 0x3186, 0x2104, 0x1082, 0x0000
+};
+
 
 
 /* Set a map for colorizing a bitmap */
 static void ColorizeBitmapSetColorMap
       (
-      RGBColorType*  color,
+      RGBColorType*  fore,
+      RGBColorType*  back,
       Boolean        invert
       )
 {
     UInt16  i;
+    if ( invert ) {
+        RGBColorType* temp;
+        temp = fore;
+        fore = back;
+        back = temp;
+    }
+    if ( fore->r == lastFore.r && fore->g == lastFore.g &&
+         fore->b == lastFore.b && back->r == lastBack.r &&
+         back->g == lastBack.g && back->b == lastBack.b ) {
+        return;
+    }
+    else {
+        lastFore = *fore;
+        lastBack = *back;
+    }
     for ( i = 0 ; i < 16 ; i++ ) {
          UInt8  r;
          UInt8  g;
          UInt8  b;
          UInt16 rgb;
-         r = ( ( ( 15 - i ) * 255 + i * ( color->r ) ) / 15 ) >> ( 8 - RED_BITS );
-         g = ( ( ( 15 - i ) * 255 + i * ( color->g ) ) / 15 ) >> ( 8 - GREEN_BITS );
-         b = ( ( ( 15 - i ) * 255 + i * ( color->b ) ) / 15 ) >> ( 8 - BLUE_BITS );
+         r = ( ( ( 15 - i ) * back->r + i * fore->r ) / 15 ) >>
+                    ( 8 - RED_BITS );
+         g = ( ( ( 15 - i ) * back->g + i * fore->g ) / 15 ) >>
+                    ( 8 - GREEN_BITS );
+         b = ( ( ( 15 - i ) * back->b + i * fore->b ) / 15 ) >>
+                    ( 8 - BLUE_BITS );
 
          rgb = ( r << ( BLUE_BITS + GREEN_BITS ) ) | ( g << BLUE_BITS ) | b;
-         if ( invert )
-             colorMap[ 15 - i ] = rgb;
-         else
-             colorMap[ i ] = rgb;
+
+         colorMap[ i ] = rgb;
     }
 }
 
 
 
-/* Colorize a 4-bit bitmap */
-/* Input bitmap: type 1 or 3.  Output: type 2 or 3 */
+/* Colorize a 4-bit bitmap into a 16-bit bitmap */
+/* Input bitmap: type 1 or 3.  Output: type 2 or 3, 16 bit */
 /* No colormaps permitted */
-static void ColorizeBitmap4
+void ColorizeBitmap4To16
       (
+      UInt16            topLeftX,
+      UInt16            topLeftY,
       PalmBitmapType*   destBitmap,
-      PalmBitmapType*   srcBitmap,
-      Boolean           srcScanLineCompress
+      PalmBitmapType*   srcBitmap
       )
 {
     UInt16  x;
@@ -182,16 +234,18 @@ static void ColorizeBitmap4
     UInt16  srcRowBytes;
     UInt16  width;
     UInt16  height;
+    Boolean scanLineCompress;
 
+    if ( srcBitmap->flags & palmCompressed )
+        scanLineCompress = true;
+    else
+        scanLineCompress = false;
     srcRowBytes        = srcBitmap->rowBytes;
     width              = srcBitmap->width;
     height             = srcBitmap->height;
-    destBitmap->width  = width;
-    destBitmap->height = height;
     destBitmap->pixelSize = 16;
+    destRowWords          = destBitmap->rowBytes / 2;
 
-    destRowWords          = width;
-    destBitmap->rowBytes = 2 * destRowWords;
     if ( srcBitmap->version < 3 ) {
         /* note that sizeof( PalmBitmapTypeV1 ) is the same as
            sizeof( PalmBitmapTypeV2 ) */
@@ -209,11 +263,20 @@ static void ColorizeBitmap4
                               sizeof( PalmBitmapTypeV3 ) );
     }
 
-    if ( ! srcScanLineCompress ) {
-        for ( y = 0 ; y < height ; y ++ ) {
-            in  = src + y * srcRowBytes;
-            out = dest + y * destRowWords;
-            for ( x = 0 ; x < width ; x += 2 ) {
+    if ( ! scanLineCompress ) {
+        UInt16 srcPaddingBytes;
+        UInt16 destIncrementWords;
+        UInt16 srcDataBytes;
+
+        srcDataBytes       = ( width + 1 ) / 2;
+        srcPaddingBytes    = srcRowBytes - srcDataBytes;
+        destIncrementWords = destRowWords - width;
+
+        in  = src;
+        out = dest + topLeftY * destRowWords + topLeftX;
+
+        for ( y = height ; 0 < y ; y -- ) {
+            for ( x = srcDataBytes ; 0 < x ; x -- ) {
                  *out++ = colorMap[ *in >> 4 ];
                  /* The next operation may overwrite the next
                     row if width is odd, but the output bitmap
@@ -222,6 +285,8 @@ static void ColorizeBitmap4
                  *out++ = colorMap[ *in & 0xF ];
                  in++;
             }
+            in  += srcPaddingBytes;
+            out += destIncrementWords;
         }
     }
     else {
@@ -239,7 +304,7 @@ static void ColorizeBitmap4
             in = src + 4;
         for ( y = 0 ; y < height ; y++ ) {
             UInt16* startOut;
-            out      = dest + y * destRowWords;
+            out      = dest + ( topLeftY + y ) * destRowWords + topLeftX;
             startOut = out;
             x        = 0;
             while ( x < srcRowNibbles ) {
@@ -271,14 +336,14 @@ static void ColorizeBitmap4
 
 
 
+
 /* Invert a 4-bit bitmap */
 /* Input bitmap: type 1 or 3.  Output bitmap: same type */
 /* No colormaps permitted */
 void InvertBitmap4
       (
       PalmBitmapType*   destBitmap,
-      PalmBitmapType*   srcBitmap,
-      Boolean           scanLineCompress
+      PalmBitmapType*   srcBitmap
       )
 {
     UInt16  x;
@@ -287,41 +352,32 @@ void InvertBitmap4
     UInt8*  out;
     UInt8*  src;
     UInt8*  dest;
-    UInt16  destRowBytes;
-    UInt16  srcRowBytes;
     UInt16  width;
     UInt16  height;
-
-    srcRowBytes        = srcBitmap->rowBytes;
-    width              = srcBitmap->width;
-    height             = srcBitmap->height;
-    destBitmap->width  = width;
-    destBitmap->height = height;
-    destBitmap->pixelSize = 4;
-
-    destRowBytes = ( width + 1 ) / 2;
-    if ( destRowBytes % 2 != 0 )
-        destRowBytes++;
-    destBitmap->rowBytes = destRowBytes;
+    UInt16  headerSize;
+    width     = srcBitmap->width;
+    height    = srcBitmap->height;
 
     if ( srcBitmap->version < 3 ) {
         /* note that sizeof( PalmBitmapTypeV1 ) is the same as
            sizeof( PalmBitmapTypeV2 ) */
-        src  = ( UInt8* )srcBitmap + sizeof( PalmBitmapTypeV1 );
-        dest = ( UInt8* )destBitmap + sizeof( PalmBitmapTypeV1 );
+        headerSize = sizeof( PalmBitmapTypeV1 );
     }
     else {
-        src  = ( UInt8* )srcBitmap + sizeof( PalmBitmapTypeV3 );
-        dest = ( UInt8* )destBitmap + sizeof( PalmBitmapTypeV3 );
-    }
+        headerSize = sizeof( PalmBitmapTypeV3 );
 
-    if ( ! scanLineCompress ) {
-        /* the following two lines work on V1, V2 and V3 */
-        ( ( PalmBitmapTypeV2* )destBitmap )->flags = 0;
-        ( ( PalmBitmapTypeV2* )destBitmap )->compressionType = 0;
+    }
+    MemMove( destBitmap, srcBitmap, headerSize );
+    src  = ( UInt8* )srcBitmap + headerSize;
+    dest = ( UInt8* )destBitmap + headerSize;
+
+    if ( ! ( srcBitmap->flags & palmCompressed ) ) {
+        UInt16  rowBytes;
+
+        rowBytes  = srcBitmap->rowBytes;
         for ( y = 0 ; y < height ; y ++ ) {
-            in  = ( UInt8* ) src + y * srcRowBytes;
-            out = ( UInt8* ) dest + y * destRowBytes;
+            in  = ( UInt8* ) src + y * rowBytes;
+            out = ( UInt8* ) dest + y * rowBytes;
             for ( x = 0 ; x < width ; x += 2 ) {
                  *out = ( ( 15 - ( *in >> 4 ) ) << 4 ) | ( 15 - ( *in & 0xF ) );
                  in++;
@@ -332,22 +388,15 @@ void InvertBitmap4
     else {
         UInt32 length;
 
-        /* the following two fields match up on V1, V2 and V3 */
-        ( ( PalmBitmapTypeV2* )destBitmap )->flags |= palmCompressed;
-        ( ( PalmBitmapTypeV2* )destBitmap )->compressionType =
-                                                  palmBitmapCompressionTypeScanLine;
-        src    = ( UInt8* )srcBitmap;
-        dest   = ( UInt8* )destBitmap;
-
         if ( 3 <= srcBitmap->version ) {
-            length = ( ( UInt32 )( ( ( UInt16* )srcBitmap )[ 0 ] ) << 16 ) |
-                     ( UInt32 )( ( ( UInt16* )srcBitmap )[ 1 ] );
+            length = ( ( UInt32 )( ( ( UInt16* )src )[ 0 ] ) << 16 ) |
+                       ( UInt32 )( ( ( UInt16* )src )[ 1 ] );
             MemMove( dest, src, 4 );
             src  += 4;
             dest += 4;
         }
         else {
-            length = ( ( UInt16* )srcBitmap )[ 0 ];
+            length = ( ( UInt16* )src )[ 0 ];
             MemMove( dest, src, 2 );
             src  += 2;
             dest += 2;
@@ -367,19 +416,42 @@ void InvertBitmap4
                     length--;
                 }
                 mask >>= 1;
-                length--;
             }
         }
     }
 }
 
 
+
+static void GrayFntClearCaches( void )
+{
+    UInt16        i;
+    UInt16        j;
+    GrayFontInfo* p;
+    for ( i = 0 ; i <= MAX_FONT_ID ; i++ ) {
+        p = grayFont[ i ];
+        if ( p != NULL ) {
+            for ( j = 0 ; j < p->header.numberOfBitmapResources ; j++ ) {
+                if ( p->resourceCache[ j ].handle != NULL ) {
+                    ReleaseFontResource( p->resourceCache[ j ].handle,
+                                         p->resourceCache[ j ].kind );
+                    p->resourceCache[ j ].handle = NULL;
+                }
+            }
+        }
+    }
+}
+
+
+
 static void GrayFntDelete
        (
        FontID font
        )
 {
-   if ( ! active || grayFont[ font ] == NULL ) {
+   GrayFontInfo* p;
+   p = grayFont[ font ];
+   if ( ! active || p == NULL ) {
        return;
    }
    if ( substitutionList[ currentFont ] == font ) {
@@ -387,11 +459,21 @@ static void GrayFntDelete
        currentFontPtr = NULL;
        FntSetFont( currentFont );
    }
-   if ( grayFont[ font ]->glyphList != NULL )
+   if ( p->glyphList != NULL )
        SafeMemPtrFree( grayFont[ font ]->glyphList );
-   if ( grayFont[ font ]->bitmapsResourceIndex != NULL )
+   if ( p->bitmapsResourceIndex != NULL )
        SafeMemPtrFree( grayFont[ font ]->bitmapsResourceIndex );
-   SafeMemPtrFree( grayFont[ font ] );
+   if ( p->resourceCache != NULL ) {
+       UInt16 i;
+       for ( i = 0 ; i < p->header.numberOfBitmapResources ; i++ ) {
+           if ( p->resourceCache[ i ].handle != NULL ) {
+               ReleaseFontResource( p->resourceCache[ i ].handle,
+                                    p->resourceCache[ i ].kind );
+           }
+       }
+       MemPtrFree( p->resourceCache );
+   }
+   SafeMemPtrFree( p );
    grayFont[ font ] = NULL;
 }
 
@@ -506,12 +588,28 @@ Err GrayFntDefineFont
     UInt16 numGlyphs;
     UInt16 newTempSize;
     UInt16 verticalLayoutSize;
+    UInt16 width;
+    UInt16 height;
+    UInt16 i;
     GrayFontInfo* newFont;
 
     GrayFntDelete( font );
     if ( * ( UInt8* ) fontP & 0x80 ) {
         return FntDefineFont( font, fontP );
     }
+    else if ( * ( UInt16* ) fontP == REMAP_FONTVERSION ) {
+        FontID mapToFontID;
+        /* Note: multiple layers of substitution are not permitted and
+           have undefined behavior, including possible crash. */
+        mapToFontID              = ( ( RemapInfoType* )fontP )->fontID;
+        substitutionList[ font ] = mapToFontID;
+        if ( currentFont == font ) {
+            currentFontPtr = grayFont[ mapToFontID ];
+        }
+        return errNone;
+    }
+    if ( ! Support35() )
+        return errOSVersionTooLow;
     grayFont[ font ] = SafeMemPtrNew( sizeof( GrayFontInfo ) );
     newFont = grayFont[ font ];
     if ( newFont == NULL )
@@ -532,6 +630,13 @@ Err GrayFntDefineFont
         GrayFntDelete( font );
         return memErrNotEnoughSpace;
     }
+    newFont->resourceCache =
+        MemPtrNew( sizeof( ResourceCacheEntry ) *
+                      newFont->header.numberOfBitmapResources );
+    if ( newFont->resourceCache == NULL ) {
+        GrayFntDelete( font );
+        return memErrNotEnoughSpace;
+    }
 
     MemMove( newFont->glyphList,
              ( UInt8* ) fontP + newFont->header.glyphInfoTableOffset,
@@ -547,10 +652,19 @@ Err GrayFntDefineFont
         currentFontPtr = newFont;
     }
 
-    newTempSize        = TempBitmapSize( newFont->header.fRectWidth,
-                              newFont->header.fRectHeight );
-    verticalLayoutSize = TempBitmapSize( newFont->header.fRectHeight,
-                              newFont->header.fRectWidth );
+    width  = newFont->header.fRectWidth;
+    height = newFont->header.fRectHeight;
+    if ( width <= MAX_SCREEN_WIDTH && height <= MAX_SCREEN_WIDTH &&
+         16 <= GetMaxBitDepth() && Have68K() ) {
+        newTempSize         = TempBitmapSize( MAX_SCREEN_WIDTH, height );
+        verticalLayoutSize  = TempBitmapSize( height, MAX_SCREEN_WIDTH );
+        fastRenderAvailable = true;
+    }
+    else {
+        newTempSize         = TempBitmapSize( width, height );
+        verticalLayoutSize  = TempBitmapSize( height, height );
+        fastRenderAvailable = false;
+    }
 
     if ( newTempSize < verticalLayoutSize )
         newTempSize = verticalLayoutSize;
@@ -565,7 +679,7 @@ Err GrayFntDefineFont
             tempBitmapSize = newTempSize;
             MemSet( tempBitmap, tempBitmapSize, 0 );
             if ( havePalmHiRes ) {
-                (( PalmBitmapTypeV3* )tempBitmap)->size    
+                (( PalmBitmapTypeV3* )tempBitmap)->size
                     = sizeof( PalmBitmapTypeV3 );
                 (( PalmBitmapTypeV3* )tempBitmap)->density
                     = PalmGetDensity();
@@ -576,6 +690,12 @@ Err GrayFntDefineFont
             }
         }
     }
+    if ( tempBitmap == NULL ) {
+        fastRenderAvailable = false;
+    }
+    for ( i = 0 ; i < newFont->header.numberOfBitmapResources ; i++ )
+        newFont->resourceCache[ i ].handle = NULL;
+
     return errNone;
 }
 
@@ -686,10 +806,10 @@ Int16 GrayFntCharsWidth
         }
     }
     else {
-        WChar  ch;
         UInt32 inOffset;
         inOffset = 0;
         while ( inOffset < length ) {
+            WChar  ch;
             inOffset += TxtGlueGetNextChar( chars, inOffset, &ch );
             if ( length < inOffset )
                 break;
@@ -709,12 +829,58 @@ Char GrayFntSetOrientation
 {
     Char oldOrientation;
     oldOrientation = resource.string[ RESOURCE_NAME_ORIENTATION ];
-    resource.string[ RESOURCE_NAME_ORIENTATION ] = o;
+    if ( oldOrientation != o ) {
+        resource.string[ RESOURCE_NAME_ORIENTATION ] = o;
+        GrayFntClearCaches();
+    }
     return oldOrientation;
 }
 
 
 
+Coord GrayFntMinLeftKerning
+       (
+       FontID font
+       )
+{
+    GrayFontInfo* fontPtr;
+    fontPtr = grayFont[ substitutionList[ font ] ];
+    if ( fontPtr == NULL ||
+         fontPtr->header.fontVersion < KERNING_FONTVERSION )
+        return 0;
+    else
+        return fontPtr->header.minLeftKerning;
+}
+
+
+
+Coord GrayFntMaxRightOverhang
+       (
+       FontID font
+       )
+{
+    GrayFontInfo* fontPtr;
+    fontPtr = grayFont[ substitutionList[ font ] ];
+    if ( fontPtr == NULL ||
+         fontPtr->header.fontVersion < KERNING_FONTVERSION )
+        return 0;
+    else
+        return fontPtr->header.maxRightOverhang;
+}
+
+
+
+
+void GrayFntSetBackgroundErase
+      (
+      Boolean state
+      )
+{
+    backgroundErase = state;
+}
+
+
+
 void GrayWinDrawCharsGeneral
       (
       Char const* chars,
@@ -724,16 +890,25 @@ void GrayWinDrawCharsGeneral
       Boolean invert
       )
 {
-    Coord*  toIncrement;
-    Int16   incrementSign;
-    UInt32  inOffset;
-    Boolean incrementFirst;
-    Boolean doMapColors;
-    Boolean doInvert;
-    UInt16  lastResourceNumber;
+    Coord*    toIncrement;
+    Int16     incrementSign;
+    UInt32    inOffset;
+    Boolean   drawBeforePosition;
+    Boolean   doMapColors;
+    Boolean   doInvert;
     MemHandle bitmapHandle   = NULL;
-    UInt8*    bitmapResource = NULL;
-    UInt16  prevCoordSys;
+    UInt16    prevCoordSys;
+    Boolean   doFastRender;
+    Coord     bitmapTopLeftX;
+    Coord     bitmapTopLeftY;
+    Coord     width;
+    Coord     oldX;
+    Coord     oldY;
+    Coord     firstKern;
+    Coord     lastOverhang;
+    WChar     ch;
+    WinDrawOperation  oldOperation = winPaint;
+    Boolean           doKern;
 
     if ( currentFontPtr == NULL ) {
         if ( invert )
@@ -742,44 +917,103 @@ void GrayWinDrawCharsGeneral
             WinDrawChars( chars, length, x, y );
         return;
     }
+    if ( length <= 0 )
+        return;
 
     HiResAdjustCurrentToNative( &x );
     HiResAdjustCurrentToNative( &y );
 
     prevCoordSys   = PalmSetCoordinateSystem( NATIVE );
 
-    switch( resource.string[ RESOURCE_NAME_ORIENTATION ] )
+    doFastRender   = false;
+    width          = 0;
+    lastOverhang   = 0;
+    if ( GrayFntMinLeftKerning( currentFont ) < 0 ||
+         0 < GrayFntMaxRightOverhang( currentFont )  ) {
+        oldOperation = WinSetDrawMode( winOverlay );
+        doKern       = true;
+    }
+    else {
+        oldOperation = WinSetDrawMode( winPaint );
+        doKern       = false;
+        if ( fastRenderAvailable && 16 == Prefs()->screenDepth ) {
+            width = GrayFntCharsWidth( chars, length );
+            doFastRender = ( width <= MAX_SCREEN_WIDTH );
+        }
+    }
+
+    bitmapTopLeftX = 0;
+    bitmapTopLeftY = 0;
+
+    TxtGlueGetNextChar( chars, 0, &ch );
+    firstKern = GetGlyph( ch )->leftKerning;
+
+    switch ( resource.string[ RESOURCE_NAME_ORIENTATION ] )
     {
         case GRAY_FONT_LEFT:
             y++;
-            toIncrement    = &y;
-            incrementSign  = -1;
-            incrementFirst = true;
+            toIncrement        = &y;
+            incrementSign      = -1;
+            drawBeforePosition = true;
+            if ( doFastRender ) {
+                bitmapTopLeftX = x;
+                bitmapTopLeftY = y - width;
+                x              = 0;
+                y              = width - firstKern;
+                tempBitmap->width    = currentFontPtr->header.fRectHeight;
+                tempBitmap->height   = width - firstKern;
+                tempBitmap->rowBytes = ( tempBitmap->width + 1 ) / 2 * 4;
+            }
             break;
         case GRAY_FONT_RIGHT:
-            toIncrement    = &y;
-            incrementSign  = 1;
-            incrementFirst = false;
+            toIncrement        = &y;
+            incrementSign      = 1;
+            drawBeforePosition = false;
             x -= currentFontPtr->header.fRectHeight;
+            if ( doFastRender ) {
+                bitmapTopLeftX = x;
+                bitmapTopLeftY = y + firstKern;
+                x              = 0;
+                y              = 0;
+                tempBitmap->width    = currentFontPtr->header.fRectHeight;
+                tempBitmap->height   = width - firstKern;
+                tempBitmap->rowBytes = ( tempBitmap->width + 1 ) / 2 * 4;
+            }
             break;
         default:
-            toIncrement    = &x;
-            incrementSign  = 1;
-            incrementFirst = false;
+            toIncrement        = &x;
+            incrementSign      = 1;
+            drawBeforePosition = false;
+            if ( doFastRender ) {
+                bitmapTopLeftX = x;
+                bitmapTopLeftY = y + firstKern;
+                x              = 0;
+                y              = 0;
+                tempBitmap->width  = width - firstKern;
+                tempBitmap->height = currentFontPtr->header.fRectHeight;
+                tempBitmap->rowBytes = ( tempBitmap->width + 1 ) / 2 * 4;
+            }
             break;
     }
     doMapColors = false;
-    doInvert    = false;
-    if ( Prefs()->screenDepth == 16 ) {
-        RGBColorType color;
-        WinSetTextColorRGB( NULL, &color );
-        if ( ( color.r != 0 || color.g != 0 || color.b != 0 || invert ) &&
-             tempBitmap != NULL ) {
-            ColorizeBitmapSetColorMap( &color, invert );
+    if ( 8 <= Prefs()->screenDepth && 16 <= GetMaxBitDepth() ) {
+        RGBColorType fore;
+        RGBColorType back;
+        WinSetTextColorRGB( NULL, &fore );
+        WinSetBackColorRGB( NULL, &back );
+        if ( doFastRender || 
+             ( ( fore.r != 0 || fore.g != 0 || fore.b != 0 ||
+                 back.r != 255 || back.g != 255 || back.b != 255 ) &&
+                                               tempBitmap != NULL ) ) {
+            ColorizeBitmapSetColorMap( &fore, &back, invert );
             if ( havePalmHiRes ) {
+                ( ( PalmBitmapTypeV2* )tempBitmap )->flags =
+                                              palmHasTransparency;
                 ( ( PalmBitmapTypeV3* )tempBitmap )->pixelFormat =
                     palmPixelFormat565;
                 tempBitmap->version = 3;
+                ( ( PalmBitmapTypeV3* )tempBitmap )->transparentValue =
+                    colorMap[ 0 ];
             }
             else {
                 tempBitmap->version = 2;
@@ -787,7 +1021,8 @@ void GrayWinDrawCharsGeneral
             doMapColors = true;
         }
     }
-    else if ( invert && tempBitmap != NULL ) {
+    doInvert    = false;
+    if ( invert && tempBitmap != NULL && ! doMapColors ) {
         doInvert = true;
         if ( havePalmHiRes ) {
             ( ( PalmBitmapTypeV3* )tempBitmap )->pixelFormat =
@@ -795,77 +1030,131 @@ void GrayWinDrawCharsGeneral
             tempBitmap->version = 3;
         }
         else {
-            tempBitmap->version = 1;
+            tempBitmap->version = 2;
         }
     }
 
     inOffset           = 0;
-    lastResourceNumber = 0;
     while ( inOffset < length ) {
         GrayFontGlyphInfo*     glyph;
-        WChar                  ch;
-        UInt16                 resourceNumber;
+        UInt16                 resourceIndex;
 
         inOffset += TxtGlueGetNextChar( chars, inOffset, &ch );
         if ( length < inOffset )
             break;
         glyph = GetGlyph( ch );
-        if ( incrementFirst )
-            *toIncrement += incrementSign * glyph->advance;
 
-        /* FIXME: handle kerning and advance correctly */
-
-        resourceNumber = glyph->resourceNumber;
-        if ( resourceNumber != lastResourceNumber ) {
-            UInt16  resourceID;
-            if ( bitmapHandle != NULL ) {
-                MemHandleUnlock( bitmapHandle );
-                DmReleaseResource( bitmapHandle );
-            }
-            resourceID     = currentFontPtr->bitmapsResourceIndex[
-                                 resourceNumber - 1 ].resourceID;
-            bitmapHandle   = DmGetResource( resource.resType, resourceID );
-            if ( bitmapHandle != NULL )
-                bitmapResource = MemHandleLock( bitmapHandle );
-            lastResourceNumber = resourceNumber;
+        resourceIndex = glyph->resourceNumber - 1;
+        if ( currentFontPtr->resourceCache[ resourceIndex ].handle != NULL ) {
+            bitmapHandle = currentFontPtr->resourceCache[ resourceIndex ].handle;
+        }
+        else {
+            UInt16   resourceID;
+            resourceID = currentFontPtr->bitmapsResourceIndex[
+                            resourceIndex ].resourceID;
+            bitmapHandle = GetFontResource( resource.resType, resourceID,
+                               &( currentFontPtr->resourceCache[
+                                                      resourceIndex ].kind ) );
+            currentFontPtr->resourceCache[ resourceIndex ].handle = bitmapHandle;
         }
 
+        oldX = x;
+        oldY = y;
+        *toIncrement += incrementSign * glyph->leftKerning;
+        if ( drawBeforePosition ) {
+            /* adjust drawing position for CCW rotation */
+            *toIncrement += incrementSign * glyph->bitmapWidth;
+        }
         if ( bitmapHandle != NULL ) {
             PalmBitmapType* bitmap;
+            RectangleType   rect;
+            UInt8*          bitmapResource;
 
+            bitmapResource = MemHandleLock( bitmapHandle );
             bitmap = ( PalmBitmapType* ) ( bitmapResource +
                         ( ( GrayFontResourceIndexEntry* )bitmapResource )[
                                  glyph->positionInResourceIndex ].offset );
-            if ( doInvert ) {
-                Boolean scanLineCompress;
-                if ( bitmap->flags & palmCompressed )
-                    scanLineCompress = true;
+            if ( doKern && ( backgroundErase || invert ) ) {
+                Int16  charRectWidth;
+                charRectWidth = glyph->bitmapWidth - lastOverhang + 
+                                    glyph->leftKerning;
+                switch ( resource.string[ RESOURCE_NAME_ORIENTATION ] ) {
+                    case GRAY_FONT_LEFT:
+                        rect.topLeft.x = x;
+                        rect.topLeft.y = y;
+                        rect.extent.x  = currentFontPtr->header.fRectHeight;
+                        rect.extent.y  = charRectWidth;
+                        break;
+                    case GRAY_FONT_RIGHT:
+                        rect.topLeft.x = x;
+                        rect.topLeft.y = y + lastOverhang - glyph->leftKerning;
+                        rect.extent.x  = currentFontPtr->header.fRectHeight;
+                        rect.extent.y  = charRectWidth;
+                        break;
+                    default:
+                        rect.topLeft.x = x + lastOverhang - glyph->leftKerning;
+                        rect.topLeft.y = y;
+                        rect.extent.x  = charRectWidth;
+                        rect.extent.y  = currentFontPtr->header.fRectHeight;
+                        break;
+                }
+                if ( invert )
+                    WinDrawRectangle( &rect, 0 );
                 else
-                    scanLineCompress = false;
-                InvertBitmap4( tempBitmap, bitmap, scanLineCompress );
-                WinDrawBitmap( ( BitmapType* )tempBitmap, x, y );
+                    WinEraseRectangle( &rect, 0 );
+            }
+            if ( doInvert ) {
+                InvertBitmap4( tempBitmap, bitmap );
+                if ( doKern ) {
+                    if ( tempBitmap->version < 3 ) {
+                        tempBitmap->version = 2;
+                        ( ( PalmBitmapTypeV2* )tempBitmap )->transparentIndex = 15;
+                    }
+                    else {
+                        ( ( PalmBitmapTypeV3* )tempBitmap )->transparentValue = 15;
+                    }
+                }
+                else
+                    tempBitmap->flags &= ~palmHasTransparency;
+                WinPaintBitmap( ( BitmapType* )tempBitmap, x, y );
             }
             else if ( doMapColors ) {
-                Boolean scanLineCompress;
-                if ( bitmap->flags & palmCompressed )
-                    scanLineCompress = true; 
-                else
-                    scanLineCompress = false;
-                ColorizeBitmap4( tempBitmap, bitmap, scanLineCompress );
-                WinDrawBitmap( ( BitmapType* )tempBitmap, x, y );
+                Coord outputX;
+                Coord outputY;
+                if ( doFastRender ) {
+                    outputX = x;
+                    outputY = y;
+                }
+                else {
+                    tempBitmap->width    = bitmap->width;
+                    tempBitmap->height   = bitmap->height;
+                    tempBitmap->rowBytes = 2 * bitmap->width;
+                    outputX              = 0;
+                    outputY              = 0;
+                }
+                ColorizeBitmap4To16( outputX, outputY, tempBitmap, bitmap );
+                if ( ! doFastRender ) {
+                    WinPaintBitmap( ( BitmapType* )tempBitmap, x, y );
+                }
             }
             else {
-                WinDrawBitmap( ( BitmapType* )bitmap, x, y );
+                WinPaintBitmap( ( BitmapType* )bitmap, x, y );
             }
+            MemHandleUnlock( bitmapHandle );
         }
-        if ( ! incrementFirst )
-            *toIncrement += incrementSign * glyph->advance;
+        x = oldX;
+        y = oldY;
+        lastOverhang = glyph->bitmapWidth - glyph->advance + glyph->leftKerning;
+        *toIncrement += incrementSign *  glyph->advance;
     }
-    if ( bitmapHandle != NULL ) {
-        MemHandleUnlock( bitmapHandle );
-        DmReleaseResource( bitmapHandle );
+    if ( doFastRender ) {
+        WinDrawBitmap( ( BitmapType* )tempBitmap, bitmapTopLeftX,
+            bitmapTopLeftY );
     }
 
+    if ( doKern ) {
+        WinSetDrawMode( oldOperation );
+    }
     PalmSetCoordinateSystem( prevCoordSys );
 }
 
@@ -905,6 +1194,9 @@ void GrayFntSubstitute
       FontID newFont
       )
 {
+    if ( grayFont[ oldFont ] != NULL ) {
+        GrayFntDelete( oldFont );
+    }
     substitutionList[ oldFont ] = newFont;
     if ( currentFont == oldFont )
         GrayFntSetFont( currentFont );
