@@ -1,5 +1,5 @@
 /*
- * $Id: hires.c,v 1.63.4.2 2003/11/13 15:08:45 prussar Exp $
+ * $Id: hires.c,v 1.67 2003/11/12 22:15:10 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -932,3 +932,36 @@ FontID HiResFont
     return hrFont;
 }
 
+
+
+#ifdef HAVE_SONY_SDK
+#undef WinDrawBitmap
+#undef WinDrawChars
+#undef WinCopyRectangle
+void HiResDrawBitmap( BitmapType* bitmapP, Coord x, Coord y )
+{
+    if ( IsHiResTypeSony( hiResType ) )
+        HRWinDrawBitmap( sonyHiResRefNum, bitmapP, x, y );
+    else
+        WinDrawBitmap( bitmapP, x, y );
+}
+
+
+void HiResDrawChars( const Char* chars, UInt16 len, Coord x, Coord y )
+{
+    if ( IsHiResTypeSony( hiResType ) )
+        HRWinDrawChars( sonyHiResRefNum, chars, len, x, y );
+    else
+        WinDrawChars( chars, len, x, y );
+}
+
+void HiResCopyRectangle( WinHandle srcWin, WinHandle dstWin, RectangleType* srcRect,
+     Coord destX, Coord destY, WinDrawOperation mode )
+{
+    if ( IsHiResTypeSony( hiResType ) )
+        HRWinCopyRectangle( sonyHiResRefNum, srcWin, dstWin, srcRect, destX, destY, mode );
+    else
+        WinCopyRectangle( srcWin, dstWin, srcRect, destX, destY, mode );
+}
+#endif
+
