@@ -1,5 +1,5 @@
 /*
- * $Id: os.c,v 1.73.2.5 2003/11/13 15:17:08 prussar Exp $
+ * $Id: os.c,v 1.87 2004/01/04 12:02:09 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -41,6 +41,10 @@
 
 #include "os.h"
 
+#ifdef HAVE_PALMCUNIT
+#include "mock_Header.h"
+#endif
+
 
 /***********************************************************************
  *
@@ -121,6 +125,7 @@ static Boolean      supportBeam;
 static Boolean      supportVFS;
 static Boolean      supportGraffiti2;
 static Boolean      supportArmlets;
+static Boolean      have68K   = true;
 static UInt32       maxBitDepth;
 static UInt32       charEncoding = charEncodingPalmLatin;
 static UInt32       notifySupport;
@@ -338,6 +343,14 @@ Boolean SupportArmlets( void )
 
 
 
+/* Do we have a 68K processor? */
+Boolean Have68K( void )
+{
+    return have68K;
+}
+
+
+
 /* Do we have a white background?  (Only supported for OS3.5+) */
 Boolean HaveWhiteBackground( void )
 {
@@ -528,7 +541,7 @@ static void OS35( void )
     OptimizeImage       = OptimizeImage_OS35;
 
     WinIndexToRGB( UIColorGetTableEntryIndex( UIFormFill ), &backColor );
-	haveWhiteBackground = ( backColor.r == 255 && backColor.g == 255 &&
+    haveWhiteBackground = ( backColor.r == 255 && backColor.g == 255 &&
                               backColor.b == 255 );
 
 #ifdef HAVE_SONY_SDK
@@ -577,11 +590,13 @@ static void OS4( void )
 /* PalmOS 5.x or higher specific operations */
 static void OS5( void )
 {
-#ifdef BUILD_ARMLETS
     Err    err;
     UInt32 processorType;
 
     err = FtrGet( sysFileCSystem, sysFtrNumProcessorID, &processorType );
+    if ( err == errNone )
+        have68K = sysFtrNumProcessorIs68K( processorType );
+#ifdef BUILD_ARMLETS
     if ( err == errNone && sysFtrNumProcessorIsARM( processorType ) )
         supportArmlets = true;
     else
@@ -614,8 +629,19 @@ Err RomVersionCompatible
 /* Initialize OS specific features */
 void OS_Init( void )
 {
+#ifdef HAVE_PALMCUNIT
+    /* When we run the test version we might call OS_Init more than
+       once during a session, so we have to reset these flags */
+    support50 = false;
+    support40 = false;
+    support35 = false;
+    support33 = false;
+    support31 = false;
+    support30 = false;
+    support20 = false;
+#endif
+
     OpenSkins();
-    InitializeUserFontDBs();
 
     FtrGet( sysFtrCreator, sysFtrNumROMVersion, &romVersion );
     if ( RomVersion20 <= romVersion ) {
@@ -651,6 +677,7 @@ void OS_Init( void )
 
         OS5();
     }
+    InitializeUserFontDBs();
 }
 
 
@@ -665,7 +692,8 @@ void OS_Release( void )
 
 #ifdef HAVE_SONY_SDK
     /* Restore original JogAssist mask */
-    HandleJogAssistMask( false );
+    if ( support35 )
+        HandleJogAssistMask( false );
 #endif
 
     ReleaseCustomFonts();
