@@ -1,5 +1,5 @@
 /*
- * $Id: prefsdata.c,v 1.83 2003/10/09 01:51:43 matto Exp $
+ * $Id: prefsdata.c,v 1.91 2004/03/07 05:29:07 prussar Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -197,6 +197,11 @@ void ReadPrefs( void )
             pref.defaultParagraphSpacing = pref.paragraphSpacing;
             pref.defaultFontModeMain     = pref.fontModeMain;
             StrCopy( pref.defaultMainUserFontName, pref.mainUserFontName );
+            pref.selectedWordAction         = SELECT_WORD_SEARCH_FORM;
+            pref.selectWordTap              = SELECT_WORD_TAP_NONE;
+            pref.hideSonyStatusBar          = false;
+            pref.useDateTime                = false;
+            pref.searchXlit                 = 0;
 
             MemSet( &pref.select, sizeof( pref.select ), SELECT_NONE );
             MemSet( &pref.hwMode, sizeof( pref.hwMode ), SELECT_NONE );
@@ -250,10 +255,10 @@ void ReadPrefs( void )
                 pref.column.date            = HIDE;
             }
             if ( old.showSize ) {
-                pref.column.date            = SHOW;
+                pref.column.size            = SHOW;
             }
             else {
-                pref.column.date            = HIDE;
+                pref.column.size            = HIDE;
             }
             pref.strikethrough              = old.strikethrough;
             pref.multipleSelect             = old.multipleSelect;
@@ -339,6 +344,11 @@ void ReadPrefs( void )
             pref.defaultParagraphSpacing = pref.paragraphSpacing;
             pref.defaultFontModeMain     = pref.fontModeMain;
             StrCopy( pref.defaultMainUserFontName, pref.mainUserFontName );
+            pref.selectedWordAction         = SELECT_WORD_SEARCH_FORM;
+            pref.selectWordTap              = SELECT_WORD_TAP_NONE;
+            pref.hideSonyStatusBar          = false;
+            pref.useDateTime                = false;
+            pref.searchXlit                 = 0;
 
             if ( sizeof( pref.select ) == sizeof( old.select ) )
                 MemMove( &pref.select, &old.select, sizeof( pref.select ) );
@@ -418,6 +428,11 @@ void ReadPrefs( void )
             StrCopy( pref.mainUserFontName, "" );
             StrCopy( pref.libraryUserFontName, "" );
             StrCopy( pref.defaultMainUserFontName, pref.mainUserFontName );
+            pref.selectedWordAction         = SELECT_WORD_SEARCH_FORM;
+            pref.selectWordTap              = SELECT_WORD_TAP_NONE;
+            pref.hideSonyStatusBar          = false;
+            pref.useDateTime                = false;
+            pref.searchXlit                 = 0;
         }
     }
 }
