@@ -1,5 +1,5 @@
 /*
- * $Id: rotate.c,v 1.22.4.1 2003/11/13 15:17:09 prussar Exp $
+ * $Id: rotate.c,v 1.28 2004/01/04 00:21:36 nordstrom Exp $
  *
  * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
  * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
@@ -40,6 +40,10 @@
 #include "armlets/rotatebitmap.h"
 #endif
 
+#ifdef HAVE_PALMCUNIT
+#include "mock_Header.h"
+#endif
+
 
 /***********************************************************************
  *
@@ -85,6 +89,12 @@ static void Rotatedxdy( Coord dx, Coord dy ) ROTATE_SECTION;
 static void RotateRectangle( RectangleType* rect ) ROTATE_SECTION;
 static Boolean InitializeRotation( void ) ROTATE_SECTION;
 static Boolean InitializeOppositeRotation( void ) ROTATE_SECTION;
+static void DrawRotatedText35( Char* string, Int16 length, Coord x,
+       Coord y, Boolean invert, Coord charTop, Coord height )
+       ROTATE_SECTION;
+static void DrawRotatedTextPre35( Char* string, Int16 length, Coord x,
+       Coord y, Boolean invert, Coord charTop, Coord height )
+       ROTATE_SECTION;
 
 
 
@@ -173,7 +183,7 @@ static Boolean InitializeRotation( void )
         case ROTATE_PLUS90:
             multiplierX = -1;
             multiplierY = 1;
-            if( IsFullscreenformActive() ) {
+            if ( IsFullscreenformActive() ) {
                 deltaX = 0;
                 deltaY = MaxExtentY() - 1;
             }
@@ -186,7 +196,7 @@ static Boolean InitializeRotation( void )
         case ROTATE_MINUS90:
             multiplierX = 1;
             multiplierY = -1;
-            if( IsFullscreenformActive() ) {
+            if ( IsFullscreenformActive() ) {
                 deltaX = MaxExtentX() - 1;
                 deltaY = 0;
             }
@@ -211,7 +221,7 @@ static Boolean InitializeOppositeRotation( void )
         case ROTATE_PLUS90:
             multiplierX = 1;
             multiplierY = -1;
-            if( IsFullscreenformActive() ) {
+            if ( IsFullscreenformActive() ) {
                 deltaX = MaxExtentY() - 1;
                 deltaY = 0;
             }
@@ -224,7 +234,7 @@ static Boolean InitializeOppositeRotation( void )
         case ROTATE_MINUS90:
             multiplierX = -1;
             multiplierY = 1;
-            if( IsFullscreenformActive() ) {
+            if ( IsFullscreenformActive() ) {
                 deltaX = 0;
                 deltaY = MaxExtentX() - 1;
             }
@@ -292,20 +302,137 @@ static UInt8 GetMaxBitDepthForBitmap
 
 
 
-
-
-
-
-/* Draw a rotated character or string at (x,y). */
-/* On OS3.5+ this calls RotBitmapMinus90 and hence only supports minus
-   90 degree rotation. */
-static void RotDrawCharsGeneral
+/* Draw rotated text on OS 3.5+ */
+void DrawRotatedText35
     (
     Char*   string,   /* string to draw */
     Int16   length,   /* length */
     Coord   x,        /* location */
     Coord   y,
-    Boolean invert    /* do we invert? */
+    Boolean invert,   /* do we invert? */
+    Coord   charTop,  /* y-offset to start drawing at */
+    Coord   height    /* y-offset to draw up to but not including */
+    )
+{
+    Int16            width;
+    WinHandle        newWindow;
+    Err              err;
+    RectangleType    bounds;
+    BitmapType*      srcBitmap;
+    BitmapType*      destBitmap;
+    UInt16           srcRowBytes;
+    UInt16           destRowBytes;
+    Boolean          clockwise;
+    Boolean          convertedSrc;
+    BitmapType*      oldSrcBitmap;
+    WinHandle        oldWindow;
+    UInt8            bitmapDepth;
+
+    clockwise = ( Prefs()->rotate == ROTATE_MINUS90 );
+
+    width = FntCharsWidth( string, length );
+
+    if ( HaveWhiteBackground() ) {
+        bitmapDepth = 1;
+    }
+    else {
+        bitmapDepth = GetMaxBitDepthForBitmap( ( UInt32 ) width *
+                          ( height - charTop ) );
+        if ( 0 == bitmapDepth )
+            return;
+    }
+
+    srcBitmap = BmpCreate( width, height - charTop, bitmapDepth, NULL, &err );
+    if ( err != errNone )
+        return;
+    destBitmap = BmpCreate( height - charTop, width, bitmapDepth, NULL, &err );
+    if ( err != errNone ) {
+        BmpDelete( srcBitmap );
+        return;
+    }
+
+    oldSrcBitmap  = srcBitmap;
+    convertedSrc  = ConvertImageToV3( &srcBitmap );
+
+    newWindow = WinCreateBitmapWindow( srcBitmap, &err );
+
+    if ( err != errNone ) {
+        BmpDelete( srcBitmap );
+        BmpDelete( destBitmap );
+        if ( convertedSrc )
+            BmpDelete( oldSrcBitmap );
+        return;
+    }
+
+    bounds.topLeft.x = 0;
+    bounds.topLeft.y = 0;
+    bounds.extent.x  = width;
+    bounds.extent.y  = height - charTop;
+    oldWindow = WinSetDrawWindow( newWindow );
+    PalmSetCoordinateSystem( NATIVE );
+    WinSetBounds( newWindow, &bounds );
+
+    if ( invert )
+        WinDrawInvertedChars( string, length, 0, -charTop );
+    else
+        WinDrawChars( string, length, 0, -charTop );
+
+    WinDeleteWindow( newWindow, false );
+    WinSetDrawWindow( oldWindow );
+
+    BmpGlueGetDimensions( srcBitmap, NULL, NULL, &srcRowBytes );
+    BmpGlueGetDimensions( destBitmap, NULL, NULL, &destRowBytes );
+
+#ifdef BUILD_ARMLETS
+    if ( RotateBitmapArmlet != NULL ) {
+        ArmRotateType armData;
+
+        armData.width        = width;
+        armData.height       = height - charTop;
+        armData.dest         = BmpGetBits( destBitmap );
+        armData.destRowBytes = destRowBytes;
+        armData.src          = BmpGetBits( srcBitmap );
+        armData.srcRowBytes  = srcRowBytes;
+        armData.clockwise    = clockwise;
+        armData.bitDepth     = bitmapDepth;
+
+        PceNativeCall( RotateBitmapArmlet, &armData );
+    }
+    else
+#endif
+        RotateBitmap( width, height - charTop,
+            BmpGetBits( destBitmap ), destRowBytes,
+            BmpGetBits( srcBitmap ), srcRowBytes,
+            clockwise, bitmapDepth );
+
+    bounds.topLeft.x = x;
+    bounds.topLeft.y = y + charTop;
+
+    RotateRectangle( &bounds );
+
+    GeneralWinDrawBitmap( destBitmap, rotatedRect.topLeft.x,
+        rotatedRect.topLeft.y );
+
+    BmpDelete( srcBitmap );
+    BmpDelete( destBitmap );
+    if ( convertedSrc )
+        BmpDelete( oldSrcBitmap );
+
+    return;
+}
+
+
+
+/* Draw rotated text on OS < 3.5 */
+void DrawRotatedTextPre35
+    (
+    Char*   string,   /* string to draw */
+    Int16   length,   /* length */
+    Coord   x,        /* location */
+    Coord   y,
+    Boolean invert,   /* do we invert? */
+    Coord   charTop,  /* y-offset to start drawing at */
+    Coord   height    /* y-offset to draw up to but not including */
     )
 {
     RectangleType     rect;
@@ -315,153 +442,15 @@ static void RotDrawCharsGeneral
     Int16             dy;
     Int16             width;
     UInt16            charX;
-    Int16             height;
     Int16             rowWords;
     UInt16*           glyph;
     UInt16*           bitmapLocationTable;
     UInt16            charNum;
     Int16             charWidth;
-    Int16             charTop;
     UInt16*           glyphPlusCharTop;
     WChar             ch;
 
-#ifdef HAVE_GRAY_FONT
-    if ( GrayFntIsCurrentGray() ) {
-        if ( Prefs()->rotate == ROTATE_MINUS90 )
-            GrayFntSetOrientation( GRAY_FONT_RIGHT );
-        else
-            GrayFntSetOrientation( GRAY_FONT_LEFT );
-        RotateXY( x, y );
-        GrayWinDrawCharsGeneral( string, length, rotatedX, rotatedY, invert );
-        return;
-    }
-#endif
-
-    height  = FntCharHeight();
-    charTop = 0;
-
-    if ( haveRange ) {
-        /* some clipping to speed things up */
-        if ( y < rangeTopY ) {
-            charTop = rangeTopY - y;
-            if ( height <= charTop )
-                return;
-        }
-        if ( rangeTopY + rangeExtentY < y + height ) {
-            height = ( rangeTopY + rangeExtentY ) - y;
-            if ( height <= charTop )
-                return;
-        }
-    }
-
-    if ( Support35() ) {
-        Int16            width;
-        WinHandle        newWindow;
-        Err              err;
-        RectangleType    bounds;
-        BitmapType*      srcBitmap;
-        BitmapType*      destBitmap;
-        UInt16           srcRowBytes;
-        UInt16           destRowBytes;
-        Boolean          clockwise;
-        Boolean          convertedSrc;
-        BitmapType*      oldSrcBitmap;
-        WinHandle        oldWindow;
-        UInt8            bitmapDepth;
-
-        clockwise = ( Prefs()->rotate == ROTATE_MINUS90 );
-
-        width = FntCharsWidth( string, length );
-        
-        if ( HaveWhiteBackground() ) {
-            bitmapDepth = 1;
-        }
-        else {
-            bitmapDepth = GetMaxBitDepthForBitmap( ( UInt32 ) width * 
-                              ( height - charTop ) );
-            if ( 0 == bitmapDepth )
-                return;
-        }
-
-        srcBitmap = BmpCreate( width, height - charTop, bitmapDepth, NULL, &err );
-        if ( err != errNone )
-            return;
-        destBitmap = BmpCreate( height - charTop, width, bitmapDepth, NULL, &err );
-        if ( err != errNone ) {
-            BmpDelete( srcBitmap );
-            return;
-        }
-
-        oldSrcBitmap  = srcBitmap;
-        convertedSrc  = ConvertImageToV3( &srcBitmap );
-
-        newWindow = WinCreateBitmapWindow( srcBitmap, &err );
-
-        if ( err != errNone ) {
-            BmpDelete( srcBitmap );
-            BmpDelete( destBitmap );
-            if ( convertedSrc )
-                BmpDelete( oldSrcBitmap );
-            return;
-        }
-
-        bounds.topLeft.x = 0;
-        bounds.topLeft.y = 0;
-        bounds.extent.x  = width;
-        bounds.extent.y  = height - charTop;
-        oldWindow = WinSetDrawWindow( newWindow );
-        PalmSetCoordinateSystem( NATIVE );
-        WinSetBounds( newWindow, &bounds );
-
-        if ( invert )
-            WinDrawInvertedChars( string, length, 0, -charTop );
-        else
-            WinDrawChars( string, length, 0, -charTop );
-
-        WinDeleteWindow( newWindow, false );
-        WinSetDrawWindow( oldWindow );
-
-        BmpGlueGetDimensions( srcBitmap, NULL, NULL, &srcRowBytes );
-        BmpGlueGetDimensions( destBitmap, NULL, NULL, &destRowBytes );
-
-#ifdef BUILD_ARMLETS
-        if ( RotateBitmapArmlet != NULL ) {
-            ArmRotateType armData;
-
-            armData.width        = width;
-            armData.height       = height - charTop;
-            armData.dest         = BmpGetBits( destBitmap );
-            armData.destRowBytes = destRowBytes;
-            armData.src          = BmpGetBits( srcBitmap );
-            armData.srcRowBytes  = srcRowBytes;
-            armData.clockwise    = clockwise;
-            armData.bitDepth     = bitmapDepth;
-
-            PceNativeCall( RotateBitmapArmlet, &armData );
-        }
-        else
-#endif
-            RotateBitmap( width, height - charTop,
-                BmpGetBits( destBitmap ), destRowBytes,
-                BmpGetBits( srcBitmap ), srcRowBytes,
-                clockwise, bitmapDepth );
-
-        bounds.topLeft.x = x;
-        bounds.topLeft.y = y + charTop;
-
-        RotateRectangle( &bounds );
-
-        GeneralWinDrawBitmap( destBitmap, rotatedRect.topLeft.x,
-            rotatedRect.topLeft.y );
-
-        BmpDelete( srcBitmap );
-        BmpDelete( destBitmap );
-        if ( convertedSrc )
-            BmpDelete( oldSrcBitmap );
-
-        return;
-    }
-
+    
     font = ( PalmFontType* ) FntGetFontPtr();
     if ( font->owTLoc < 0 )
         return;
@@ -503,11 +492,11 @@ static void RotDrawCharsGeneral
 
         width = owTable[ charNum ].width;
 
-         rect.topLeft.x   = x;
-         rect.extent.x    = width;
+        rect.topLeft.x   = x;
+        rect.extent.x    = width;
 
-         RotateRectangle( &rect );
-         if ( ! invert )
+        RotateRectangle( &rect );
+        if ( ! invert )
             WinEraseRectangle( &rotatedRect, 0 );
         else
             WinDrawRectangle( &rotatedRect, 0 );
@@ -541,11 +530,66 @@ static void RotDrawCharsGeneral
         }
 
         x += width;
-    } 
+    }
 }
 
 
 
+/* Draw a rotated character or string at (x,y). */
+/* On OS3.5+ this calls RotBitmapMinus90 and hence only supports minus
+   90 degree rotation. */
+static void RotDrawCharsGeneral
+    (
+    Char*   string,   /* string to draw */
+    Int16   length,   /* length */
+    Coord   x,        /* location */
+    Coord   y,
+    Boolean invert    /* do we invert? */
+    )
+{
+    Coord   charTop;
+    Coord   height;
+
+#ifdef HAVE_GRAY_FONT
+    if ( GrayFntIsCurrentGray() ) {
+        if ( Prefs()->rotate == ROTATE_MINUS90 )
+            GrayFntSetOrientation( GRAY_FONT_RIGHT );
+        else
+            GrayFntSetOrientation( GRAY_FONT_LEFT );
+        RotateXY( x, y );
+        GrayWinDrawCharsGeneral( string, length, rotatedX, rotatedY, invert );
+        return;
+    }
+#endif
+
+    height  = FntCharHeight();
+    charTop = 0;
+
+    if ( haveRange ) {
+        /* some clipping to speed things up */
+        if ( y < rangeTopY ) {
+            charTop = rangeTopY - y;
+            if ( height <= charTop )
+                return;
+        }
+        if ( rangeTopY + rangeExtentY < y + height ) {
+            height = ( rangeTopY + rangeExtentY ) - y;
+            if ( height <= charTop )
+                return;
+        }
+    }
+
+    if ( Support35() ) {
+        DrawRotatedText35( string, length, x, y, invert, charTop, height );
+    }
+    else {
+        DrawRotatedTextPre35( string, length, x, y, invert, charTop, height );
+    }
+}
+
+
+
+
 /* We include replacements for a number of PalmOS functions, and
    a few Plucker ones, that can be used instead of the originals.
    Input coordinates, the ones that the rest of Plucker works with, 
@@ -1066,7 +1110,7 @@ void RotateArrowKeys
         }
     }
     else {
-        switch( *key ) {
+        switch ( *key ) {
             case chrDownArrow:
                  *key = chrRightArrow;
                  break;
